@@ -49,6 +49,8 @@
 
 #include <gtk/gtk.h>
 
+#include <exo/exo.h>
+
 #include <libxfce4util/libxfce4util.h>
 
 #include <thunar-uca/thunar-uca-model.h>
@@ -122,7 +124,7 @@ static gboolean           thunar_uca_model_load_from_file   (ThunarUcaModel
                                                              const gchar          *filename,
                                                              GError              **error);
 static void               thunar_uca_model_item_reset       (ThunarUcaModelItem   *item);
-static void               thunar_uca_model_item_free        (ThunarUcaModelItem   *item);
+static void               thunar_uca_model_item_free        (gpointer              data);
 static void               start_element_handler             (GMarkupParseContext  *context,
                                                              const gchar          *element_name,
                                                              const gchar         **attribute_names,
@@ -160,7 +162,8 @@ struct _ThunarUcaModelItem
 {
   gchar         *name;
   gchar         *description;
-  gchar         *icon;
+  gchar         *icon_name;
+  GIcon         *gicon;
   gchar         *command;
   guint          startup_notify : 1;
   gchar        **patterns;
@@ -176,11 +179,11 @@ typedef struct
 {
   ParserStack    *stack;
   ThunarUcaModel *model;
-  const gchar    *locale;
+  gchar          *locale;
   GString        *name;
   gboolean        name_use;
   guint           name_match;
-  GString        *icon;
+  GString        *icon_name;
   GString        *command;
   GString        *patterns;
   GString        *description;
@@ -284,8 +287,7 @@ thunar_uca_model_finalize (GObject *object)
   g_object_unref (G_OBJECT (uca_model->icon_factory));
 
   /* release all items */
-  g_list_foreach (uca_model->items, (GFunc) thunar_uca_model_item_free, NULL);
-  g_list_free (uca_model->items);
+  g_list_free_full (uca_model->items, thunar_uca_model_item_free);
 
   (*G_OBJECT_CLASS (thunar_uca_model_parent_class)->finalize) (object);
 }
@@ -320,7 +322,10 @@ thunar_uca_model_get_column_type (GtkTreeModel *tree_model,
     case THUNAR_UCA_MODEL_COLUMN_DESCRIPTION:
       return G_TYPE_STRING;
 
-    case THUNAR_UCA_MODEL_COLUMN_ICON:
+    case THUNAR_UCA_MODEL_COLUMN_GICON:
+      return G_TYPE_ICON;
+
+    case THUNAR_UCA_MODEL_COLUMN_ICON_NAME:
       return G_TYPE_STRING;
 
     case THUNAR_UCA_MODEL_COLUMN_COMMAND:
@@ -410,8 +415,18 @@ thunar_uca_model_get_value (GtkTreeModel *tree_model,
       g_value_set_static_string (value, item->description);
       break;
 
-    case THUNAR_UCA_MODEL_COLUMN_ICON:
-      g_value_set_static_string (value, item->icon);
+    case THUNAR_UCA_MODEL_COLUMN_GICON:
+      if (item->gicon == NULL && item->icon_name != NULL)
+        {
+          /* cache gicon from the name */
+          item->gicon = g_icon_new_for_string (item->icon_name, NULL);
+        }
+
+      g_value_set_object (value, item->gicon);
+      break;
+
+    case THUNAR_UCA_MODEL_COLUMN_ICON_NAME:
+      g_value_set_static_string (value, item->icon_name);
       break;
 
     case THUNAR_UCA_MODEL_COLUMN_COMMAND:
@@ -553,9 +568,9 @@ thunar_uca_model_load_from_file (ThunarUcaModel *uca_model,
   /* initialize the parser */
   parser.stack = xfce_stack_new (ParserStack);
   parser.model = uca_model;
-  parser.locale = setlocale (LC_MESSAGES, NULL);
+  parser.locale = g_strdup (setlocale (LC_MESSAGES, NULL));
   parser.name = g_string_new (NULL);
-  parser.icon = g_string_new (NULL);
+  parser.icon_name = g_string_new (NULL);
   parser.command = g_string_new (NULL);
   parser.patterns = g_string_new (NULL);
   parser.description = g_string_new (NULL);
@@ -572,8 +587,9 @@ thunar_uca_model_load_from_file (ThunarUcaModel *uca_model,
   g_string_free (parser.description, TRUE);
   g_string_free (parser.patterns, TRUE);
   g_string_free (parser.command, TRUE);
-  g_string_free (parser.icon, TRUE);
+  g_string_free (parser.icon_name, TRUE);
   g_string_free (parser.name, TRUE);
+  g_free (parser.locale);
   xfce_stack_free (parser.stack);
   g_free (content);
 
@@ -590,7 +606,10 @@ thunar_uca_model_item_reset (ThunarUcaModelItem *item)
   g_free (item->description);
   g_free (item->command);
   g_free (item->name);
-  g_free (item->icon);
+  g_free (item->icon_name);
+
+  if (item->gicon != NULL)
+    g_object_unref (item->gicon);
 
   /* ...and reset the item memory */
   memset (item, 0, sizeof (*item));
@@ -599,8 +618,9 @@ thunar_uca_model_item_reset (ThunarUcaModelItem *item)
 
 
 static void
-thunar_uca_model_item_free (ThunarUcaModelItem *item)
+thunar_uca_model_item_free (gpointer data)
 {
+  ThunarUcaModelItem *item = data;
   thunar_uca_model_item_reset (item);
   g_free (item);
 }
@@ -634,7 +654,7 @@ start_element_handler (GMarkupParseContext *context,
           parser->description_match = XFCE_LOCALE_NO_MATCH;
           parser->types = 0;
           parser->startup_notify = FALSE;
-          g_string_truncate (parser->icon, 0);
+          g_string_truncate (parser->icon_name, 0);
           g_string_truncate (parser->name, 0);
           g_string_truncate (parser->command, 0);
           g_string_truncate (parser->patterns, 0);
@@ -677,7 +697,7 @@ start_element_handler (GMarkupParseContext *context,
         }
       else if (strcmp (element_name, "icon") == 0)
         {
-          g_string_truncate (parser->icon, 0);
+          g_string_truncate (parser->icon_name, 0);
           xfce_stack_push (parser->stack, PARSER_ICON);
         }
       else if (strcmp (element_name, "command") == 0)
@@ -799,7 +819,7 @@ end_element_handler (GMarkupParseContext *context,
           thunar_uca_model_update (parser->model, &iter,
                                    parser->name->str,
                                    parser->description->str,
-                                   parser->icon->str,
+                                   parser->icon_name->str,
                                    parser->command->str,
                                    parser->startup_notify,
                                    parser->patterns->str,
@@ -900,7 +920,7 @@ text_handler (GMarkupParseContext *context,
       break;
 
     case PARSER_ICON:
-      g_string_append_len (parser->icon, text, text_len);
+      g_string_append_len (parser->icon_name, text, text_len);
       break;
 
     case PARSER_COMMAND:
@@ -999,8 +1019,7 @@ types_from_mime_type (const gchar *mime_type)
  *
  * The caller is responsible to free the returned list using
  * <informalexample><programlisting>
- * g_list_foreach (list, (GFunc) gtk_tree_path_free, NULL);
- * g_list_free (list);
+ * g_list_free_full (list, (GDestroyNotify) gtk_tree_path_free);
  * </programlisting></informalexample>
  *
  * Return value: the list of #GtkTreePath<!---->s to items
@@ -1257,7 +1276,7 @@ thunar_uca_model_update (ThunarUcaModel *uca_model,
   if (G_LIKELY (name != NULL && *name != '\0'))
     item->name = g_strdup (name);
   if (G_LIKELY (icon != NULL && *icon != '\0'))
-    item->icon = g_strdup (icon);
+    item->icon_name = g_strdup (icon);
   if (G_LIKELY (command != NULL && *command != '\0'))
     item->command = g_strdup (command);
   if (G_LIKELY (description != NULL && *description != '\0'))
@@ -1338,20 +1357,20 @@ thunar_uca_model_save (ThunarUcaModel *uca_model,
   fp = fdopen (fd, "w");
 
   /* write the header */
-  fprintf (fp, "<?xml encoding=\"UTF-8\" version=\"1.0\"?>\n<actions>");
+  fprintf (fp, "<?xml encoding=\"UTF-8\" version=\"1.0\"?>\n<actions>\n");
 
   /* write the model items */
   for (lp = uca_model->items; lp != NULL; lp = lp->next)
     {
       item = (ThunarUcaModelItem *) lp->data;
-      fprintf (fp, "<action>");
+      fprintf (fp, "<action>\n");
       patterns = g_strjoinv (";", item->patterns);
-      escaped = g_markup_printf_escaped ("<icon>%s</icon>"
-                                         "<name>%s</name>"
-                                         "<command>%s</command>"
-                                         "<description>%s</description>"
-                                         "<patterns>%s</patterns>",
-                                         (item->icon != NULL) ? item->icon : "",
+      escaped = g_markup_printf_escaped ("\t<icon>%s</icon>\n"
+                                         "\t<name>%s</name>\n"
+                                         "\t<command>%s</command>\n"
+                                         "\t<description>%s</description>\n"
+                                         "\t<patterns>%s</patterns>\n",
+                                         (item->icon_name != NULL) ? item->icon_name : "",
                                          (item->name != NULL) ? item->name : "",
                                          (item->command != NULL) ? item->command : "",
                                          (item->description != NULL) ? item->description : "",
@@ -1360,20 +1379,20 @@ thunar_uca_model_save (ThunarUcaModel *uca_model,
       g_free (patterns);
       g_free (escaped);
       if (item->startup_notify)
-        fprintf (fp, "<startup-notify/>");
+        fprintf (fp, "\t<startup-notify/>\n");
       if ((item->types & THUNAR_UCA_TYPE_DIRECTORIES) != 0)
-        fprintf (fp, "<directories/>");
+        fprintf (fp, "\t<directories/>\n");
       if ((item->types & THUNAR_UCA_TYPE_AUDIO_FILES) != 0)
-        fprintf (fp, "<audio-files/>");
+        fprintf (fp, "\t<audio-files/>\n");
       if ((item->types & THUNAR_UCA_TYPE_IMAGE_FILES) != 0)
-        fprintf (fp, "<image-files/>");
+        fprintf (fp, "\t<image-files/>\n");
       if ((item->types & THUNAR_UCA_TYPE_OTHER_FILES) != 0)
-        fprintf (fp, "<other-files/>");
+        fprintf (fp, "\t<other-files/>\n");
       if ((item->types & THUNAR_UCA_TYPE_TEXT_FILES) != 0)
-        fprintf (fp, "<text-files/>");
+        fprintf (fp, "\t<text-files/>\n");
       if ((item->types & THUNAR_UCA_TYPE_VIDEO_FILES) != 0)
-        fprintf (fp, "<video-files/>");
-      fprintf (fp, "</action>");
+        fprintf (fp, "\t<video-files/>\n");
+      fprintf (fp, "</action>\n");
     }
 
   /* write the footer and close the tmp file */
