@@ -56,6 +56,7 @@
 #include <thunar/thunar-renamer-dialog.h>
 #include <thunar/thunar-thumbnail-cache.h>
 #include <thunar/thunar-util.h>
+#include <thunar/thunar-view.h>
 
 
 
@@ -91,6 +92,8 @@ static void           thunar_application_collect_and_launch     (ThunarApplicati
                                                                  GList                  *source_file_list,
                                                                  GFile                  *target_file,
                                                                  GClosure               *new_files_closure);
+static void           thunar_application_launch_finished        (ThunarJob              *job,
+                                                                 ThunarView             *view);
 static void           thunar_application_launch                 (ThunarApplication      *application,
                                                                  gpointer                parent,
                                                                  const gchar            *icon_name,
@@ -115,6 +118,7 @@ static void           thunar_application_volman_watch_destroy   (gpointer
 #endif
 static gboolean       thunar_application_show_dialogs           (gpointer                user_data);
 static void           thunar_application_show_dialogs_destroy   (gpointer                user_data);
+static GtkWidget     *thunar_application_get_progress_dialog    (ThunarApplication      *application);
 static void           thunar_application_process_files          (ThunarApplication      *application);
 
 
@@ -265,8 +269,7 @@ thunar_application_finalize (GObject *object)
     g_source_remove (application->volman_idle_id);
 
   /* drop all pending volume manager UDIs */
-  g_slist_foreach (application->volman_udis, (GFunc) g_free, NULL);
-  g_slist_free (application->volman_udis);
+  g_slist_free_full (application->volman_udis, g_free);
 
   /* disconnect from the udev client */
   g_object_unref (application->udev_client);
@@ -402,6 +405,42 @@ thunar_application_collect_and_launch (ThunarApplication *application,
 
 
 
+static void
+thunar_application_launch_finished_too_late (gpointer  user_data,
+                                             GObject  *where_the_object_was)
+{
+  ThunarJob *job = THUNAR_JOB (user_data);
+
+  _thunar_return_if_fail (THUNAR_IS_JOB (job));
+  _thunar_return_if_fail (THUNAR_IS_VIEW (where_the_object_was));
+
+  /* remove the finished signal */
+  g_signal_handlers_disconnect_by_func (G_OBJECT (job),
+                                        G_CALLBACK (thunar_application_launch_finished),
+                                        where_the_object_was);
+}
+
+
+
+static void
+thunar_application_launch_finished (ThunarJob  *job,
+                                    ThunarView *view)
+{
+  _thunar_return_if_fail (THUNAR_IS_JOB (job));
+  _thunar_return_if_fail (THUNAR_IS_VIEW (view));
+
+  /* remove the view weak ref */
+  g_object_weak_unref (G_OBJECT (view),
+                       thunar_application_launch_finished_too_late,
+                       job);
+
+  /* the job completed, refresh the interface
+   * directly to make it feel snappy */
+  thunar_view_reload (view);
+}
+
+
+
 static void
 thunar_application_launch (ThunarApplication *application,
                            gpointer           parent,
@@ -423,7 +462,21 @@ thunar_application_launch (ThunarApplication *application,
 
   /* try to allocate a new job for the operation */
   job = (*launcher) (source_file_list, target_file_list);
-    
+
+  if (THUNAR_IS_VIEW (parent))
+    {
+      /* connect a callback to instantly refresh the thunar view */
+      g_signal_connect (G_OBJECT (job), "finished",
+                        G_CALLBACK (thunar_application_launch_finished),
+                        parent);
+
+      /* watch destruction of the parent, so we disconnect before the
+       * job is finished */
+      g_object_weak_ref (G_OBJECT (parent),
+                         thunar_application_launch_finished_too_late,
+                         job);
+    }
+
   /* connect the "new-files" closure (if any) */
   if (G_LIKELY (new_files_closure != NULL))
     g_signal_connect_closure (job, "new-files", new_files_closure, FALSE);
@@ -606,8 +659,7 @@ thunar_application_volman_idle (gpointer user_data)
   else
     {
       /* drop all pending HAL device UDIs */
-      g_slist_foreach (application->volman_udis, (GFunc) g_free, NULL);
-      g_slist_free (application->volman_udis);
+      g_slist_free_full (application->volman_udis, g_free);
       application->volman_udis = NULL;
     }
 
@@ -995,7 +1047,7 @@ thunar_application_bulk_rename (ThunarApplication *application,
 
 
 
-GtkWidget *
+static GtkWidget *
 thunar_application_get_progress_dialog (ThunarApplication *application)
 {
   _thunar_return_val_if_fail (THUNAR_IS_APPLICATION (application), NULL);
@@ -1363,7 +1415,7 @@ thunar_application_create_file (ThunarApplication *application,
       if (is_directory)
         thunar_application_mkdir (application, screen, &path_list, NULL);
       else
-        thunar_application_creat (application, screen, &path_list, NULL);
+        thunar_application_creat (application, screen, &path_list, NULL, NULL);
 
       g_object_unref (path_list.data);
       g_free (name);
@@ -1393,7 +1445,6 @@ thunar_application_create_file_from_template (ThunarApplication *application,
                                               GdkScreen         *screen,
                                               const gchar       *startup_id)
 {
-  GList  source_path_list;
   GList  target_path_list;
   gchar *name;
   gchar *title;
@@ -1417,18 +1468,15 @@ thunar_application_create_file_from_template (ThunarApplication *application,
                                     title);
   if (G_LIKELY (name != NULL))
     {
-      /* fake the source file list */
-      source_path_list.data = thunar_file_get_file (template_file);
-      source_path_list.prev = source_path_list.next = NULL;
-
       /* fake the target path list */
       target_path_list.data = g_file_get_child (thunar_file_get_file (parent_directory), name);
       target_path_list.next = target_path_list.prev = NULL;
 
       /* launch the operation */
-      thunar_application_copy_to (application, screen, 
-                                  &source_path_list, &target_path_list,
-                                  NULL);
+      thunar_application_creat (application, screen,
+                                &target_path_list,
+                                thunar_file_get_file (template_file),
+                                NULL);
 
       /* release the target path */
       g_object_unref (target_path_list.data);
@@ -1638,9 +1686,7 @@ unlink_stub (GList *source_path_list,
  * @application : a #ThunarApplication.
  * @parent      : a #GdkScreen, a #GtkWidget or %NULL.
  * @file_list   : the list of #ThunarFile<!---->s that should be deleted.
- * @permanently : whether to unlink the files permanently. Even if this is set to
- *                FALSE, the files may be erased permanently if the SHIFT key
- *                is pressed at the time the function is called.
+ * @permanently : whether to unlink the files permanently.
  *
  * Deletes all files in the @file_list and takes care of all user interaction.
  *
@@ -1654,25 +1700,18 @@ thunar_application_unlink_files (ThunarApplication *application,
                                  GList             *file_list,
                                  gboolean           permanently)
 {
-  GdkModifierType state;
-  GtkWidget      *dialog;
-  GtkWindow      *window;
-  GdkScreen      *screen;
-  GList          *path_list = NULL;
-  GList          *lp;
-  gchar          *message;      
-  guint           n_path_list = 0;
-  gint            response;
+  GtkWidget *dialog;
+  GtkWindow *window;
+  GdkScreen *screen;
+  GList     *path_list = NULL;
+  GList     *lp;
+  gchar     *message;
+  guint      n_path_list = 0;
+  gint       response;
 
   _thunar_return_if_fail (parent == NULL || GDK_IS_SCREEN (parent) || GTK_IS_WIDGET (parent));
   _thunar_return_if_fail (THUNAR_IS_APPLICATION (application));
 
-  if (!permanently)
-    {
-      /* check if we should permanently delete the files (user holds shift) */
-      permanently = (gtk_get_current_event_state (&state) && (state & GDK_SHIFT_MASK) != 0);
-    }
-
   /* determine the paths for the files */
   for (lp = g_list_last (file_list); lp != NULL; lp = lp->prev, ++n_path_list)
     {
@@ -1775,10 +1814,11 @@ thunar_application_trash (ThunarApplication *application,
 
 
 static ThunarJob *
-creat_stub (GList *source_path_list,
+creat_stub (GList *template_file,
             GList *target_path_list)
 {
-  return thunar_io_jobs_create_files (source_path_list);
+   _thunar_return_val_if_fail (template_file->data == NULL || G_IS_FILE (template_file->data), NULL);
+  return thunar_io_jobs_create_files (target_path_list, template_file->data);
 }
 
 
@@ -1800,15 +1840,21 @@ void
 thunar_application_creat (ThunarApplication *application,
                           gpointer           parent,
                           GList             *file_list,
+                          GFile             *template_file,
                           GClosure          *new_files_closure)
 {
+  GList template_list;
+
   _thunar_return_if_fail (parent == NULL || GDK_IS_SCREEN (parent) || GTK_IS_WIDGET (parent));
   _thunar_return_if_fail (THUNAR_IS_APPLICATION (application));
   
+  template_list.next = template_list.prev = NULL;
+  template_list.data = template_file;
+
   /* launch the operation */
   thunar_application_launch (application, parent, "document-new",
                              _("Creating files..."), creat_stub,
-                             file_list, file_list, new_files_closure);
+                             &template_list, file_list, new_files_closure);
 }
 
 
