@@ -206,7 +206,7 @@ thunar_chooser_button_get_property (GObject    *object,
   switch (prop_id)
     {
     case PROP_FILE:
-      g_value_set_object (value, thunar_chooser_button_get_file (chooser_button));
+      g_value_set_object (value, chooser_button->file);
       break;
 
     default:
@@ -453,21 +453,25 @@ thunar_chooser_button_file_changed (ThunarChooserButton *chooser_button,
           /* add all possible applications */
           for (lp = app_infos, i = 0; lp != NULL; lp = lp->next, ++i)
             {
-              /* insert the item into the store */
-              gtk_list_store_insert_with_values (chooser_button->store, &iter, i,
-                                                 THUNAR_CHOOSER_BUTTON_STORE_COLUMN_NAME,
-                                                 g_app_info_get_name (lp->data),
-                                                 THUNAR_CHOOSER_BUTTON_STORE_COLUMN_APPLICATION,
-                                                 lp->data,
-                                                 THUNAR_CHOOSER_BUTTON_STORE_COLUMN_ICON,
-                                                 g_app_info_get_icon (lp->data),
-                                                 THUNAR_CHOOSER_BUTTON_STORE_COLUMN_SENSITIVE,
-                                                 TRUE,
-                                                 -1);
-
-              /* pre-select the default application */
-              if (g_app_info_equal (lp->data, app_info))
-                gtk_combo_box_set_active_iter (GTK_COMBO_BOX (chooser_button), &iter);
+              /* skip infos that have NoDisplay or OnlyShowIn set */
+              if (g_app_info_should_show (lp->data))
+                {
+                  /* insert the item into the store */
+                  gtk_list_store_insert_with_values (chooser_button->store, &iter, i,
+                                                     THUNAR_CHOOSER_BUTTON_STORE_COLUMN_NAME,
+                                                     g_app_info_get_name (lp->data),
+                                                     THUNAR_CHOOSER_BUTTON_STORE_COLUMN_APPLICATION,
+                                                     lp->data,
+                                                     THUNAR_CHOOSER_BUTTON_STORE_COLUMN_ICON,
+                                                     g_app_info_get_icon (lp->data),
+                                                     THUNAR_CHOOSER_BUTTON_STORE_COLUMN_SENSITIVE,
+                                                     TRUE,
+                                                     -1);
+                  
+                  /* pre-select the default application */
+                  if (g_app_info_equal (lp->data, app_info))
+                    gtk_combo_box_set_active_iter (GTK_COMBO_BOX (chooser_button), &iter);
+                }
 
               /* release the application */
               g_object_unref (lp->data);
@@ -530,23 +534,6 @@ thunar_chooser_button_new (void)
 
 
 
-/**
- * thunar_chooser_button_get_file:
- * @chooser_button : a #ThunarChooserButton instance.
- *
- * Returns the #ThunarFile associated with @chooser_button.
- *
- * Return value: the file associated with @chooser_button.
- **/
-ThunarFile*
-thunar_chooser_button_get_file (ThunarChooserButton *chooser_button)
-{
-  _thunar_return_val_if_fail (THUNAR_IS_CHOOSER_BUTTON (chooser_button), NULL);
-  return chooser_button->file;
-}
-
-
-
 /**
  * thunar_chooser_button_set_file:
  * @chooser_button : a #ThunarChooserButton instance.
