@@ -1,6 +1,7 @@
-/* $Id: thunar-chooser-dialog.c 26421 2007-12-02 13:46:28Z benny $ */
+/* $Id$ */
 /*-
- * Copyright (c) 2005-2006 Benedikt Meurer <benny@xfce.org>.
+ * Copyright (c) 2005-2006 Benedikt Meurer <benny@xfce.org>
+ * Copyright (c) 2009 Jannis Pohlmann <jannis@xfce.org>
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the Free
@@ -50,51 +51,52 @@ enum
 
 
 
-static void     thunar_chooser_dialog_class_init          (ThunarChooserDialogClass *klass);
-static void     thunar_chooser_dialog_init                (ThunarChooserDialog      *dialog);
-static void     thunar_chooser_dialog_dispose             (GObject                  *object);
-static void     thunar_chooser_dialog_get_property        (GObject                  *object,
-                                                           guint                     prop_id,
-                                                           GValue                   *value,
-                                                           GParamSpec               *pspec);
-static void     thunar_chooser_dialog_set_property        (GObject                  *object,
-                                                           guint                     prop_id,
-                                                           const GValue             *value,
-                                                           GParamSpec               *pspec);
-static void     thunar_chooser_dialog_realize             (GtkWidget                *widget);
-static void     thunar_chooser_dialog_response            (GtkDialog                *widget,
-                                                           gint                      response);
-static gboolean thunar_chooser_dialog_selection_func      (GtkTreeSelection         *selection,
-                                                           GtkTreeModel             *model,
-                                                           GtkTreePath              *path,
-                                                           gboolean                  path_currently_selected,
-                                                           gpointer                  user_data);
-static gboolean thunar_chooser_dialog_context_menu        (ThunarChooserDialog      *dialog,
-                                                           guint                     button,
-                                                           guint32                   time);
-static void     thunar_chooser_dialog_update_accept       (ThunarChooserDialog      *dialog);
-static void     thunar_chooser_dialog_update_header       (ThunarChooserDialog      *dialog);
-static void     thunar_chooser_dialog_action_remove       (ThunarChooserDialog      *dialog);
-static void     thunar_chooser_dialog_browse_clicked      (GtkWidget                *button,
-                                                           ThunarChooserDialog      *dialog);
-static gboolean thunar_chooser_dialog_button_press_event  (GtkWidget                *tree_view,
-                                                           GdkEventButton           *event,
-                                                           ThunarChooserDialog      *dialog);
-static void     thunar_chooser_dialog_notify_expanded     (GtkExpander              *expander,
-                                                           GParamSpec               *pspec,
-                                                           ThunarChooserDialog      *dialog);
-static void     thunar_chooser_dialog_notify_loading      (ThunarChooserModel       *model,
-                                                           GParamSpec               *pspec,
-                                                           ThunarChooserDialog      *dialog);
-static gboolean thunar_chooser_dialog_popup_menu          (GtkWidget                *tree_view,
-                                                           ThunarChooserDialog      *dialog);
-static void     thunar_chooser_dialog_row_activated       (GtkTreeView              *treeview,
-                                                           GtkTreePath              *path,
-                                                           GtkTreeViewColumn        *column,
-                                                           ThunarChooserDialog      *dialog);
-static void     thunar_chooser_dialog_selection_changed   (GtkTreeSelection         *selection,
-                                                           ThunarChooserDialog      *dialog);
-
+static void        thunar_chooser_dialog_dispose             (GObject                  *object);
+static void        thunar_chooser_dialog_get_property        (GObject                  *object,
+                                                              guint                     prop_id,
+                                                              GValue                   *value,
+                                                              GParamSpec               *pspec);
+static void        thunar_chooser_dialog_set_property        (GObject                  *object,
+                                                              guint                     prop_id,
+                                                              const GValue             *value,
+                                                              GParamSpec               *pspec);
+static void        thunar_chooser_dialog_realize             (GtkWidget                *widget);
+static void        thunar_chooser_dialog_response            (GtkDialog                *widget,
+                                                              gint                      response);
+static gboolean    thunar_chooser_dialog_selection_func      (GtkTreeSelection         *selection,
+                                                              GtkTreeModel             *model,
+                                                              GtkTreePath              *path,
+                                                              gboolean                  path_currently_selected,
+                                                              gpointer                  user_data);
+static gboolean    thunar_chooser_dialog_context_menu        (ThunarChooserDialog      *dialog,
+                                                              guint                     button,
+                                                              guint32                   time);
+static void        thunar_chooser_dialog_update_accept       (ThunarChooserDialog      *dialog);
+static void        thunar_chooser_dialog_update_header       (ThunarChooserDialog      *dialog);
+static void        thunar_chooser_dialog_action_remove       (ThunarChooserDialog      *dialog);
+static void        thunar_chooser_dialog_browse_clicked      (GtkWidget                *button,
+                                                              ThunarChooserDialog      *dialog);
+static gboolean    thunar_chooser_dialog_button_press_event  (GtkWidget                *tree_view,
+                                                              GdkEventButton           *event,
+                                                              ThunarChooserDialog      *dialog);
+static void        thunar_chooser_dialog_notify_expanded     (GtkExpander              *expander,
+                                                              GParamSpec               *pspec,
+                                                              ThunarChooserDialog      *dialog);
+static void        thunar_chooser_dialog_expand              (ThunarChooserDialog      *dialog);
+static gboolean    thunar_chooser_dialog_popup_menu          (GtkWidget                *tree_view,
+                                                              ThunarChooserDialog      *dialog);
+static void        thunar_chooser_dialog_row_activated       (GtkTreeView              *treeview,
+                                                              GtkTreePath              *path,
+                                                              GtkTreeViewColumn        *column,
+                                                              ThunarChooserDialog      *dialog);
+static void        thunar_chooser_dialog_selection_changed   (GtkTreeSelection         *selection,
+                                                              ThunarChooserDialog      *dialog);
+static ThunarFile *thunar_chooser_dialog_get_file            (ThunarChooserDialog *dialog);
+static void        thunar_chooser_dialog_set_file            (ThunarChooserDialog *dialog,
+                                                              ThunarFile          *file);
+static gboolean    thunar_chooser_dialog_get_open            (ThunarChooserDialog *dialog);
+static void        thunar_chooser_dialog_set_open            (ThunarChooserDialog *dialog,
+                                                              gboolean             open);
 
 
 struct _ThunarChooserDialogClass
@@ -122,36 +124,7 @@ struct _ThunarChooserDialog
 
 
 
-static GObjectClass *thunar_chooser_dialog_parent_class;
-
-
-
-GType
-thunar_chooser_dialog_get_type (void)
-{
-  static GType type = G_TYPE_INVALID;
-
-  if (G_UNLIKELY (type == G_TYPE_INVALID))
-    {
-      static const GTypeInfo info =
-      {
-        sizeof (ThunarChooserDialogClass),
-        NULL,
-        NULL,
-        (GClassInitFunc) thunar_chooser_dialog_class_init,
-        NULL,
-        NULL,
-        sizeof (ThunarChooserDialog),
-        0,
-        (GInstanceInitFunc) thunar_chooser_dialog_init,
-        NULL,
-      };
-
-      type = g_type_register_static (THUNAR_TYPE_ABSTRACT_DIALOG, I_("ThunarChooserDialog"), &info, 0);
-    }
-
-  return type;
-}
+G_DEFINE_TYPE (ThunarChooserDialog, thunar_chooser_dialog, THUNAR_TYPE_ABSTRACT_DIALOG)
 
 
 
@@ -162,9 +135,6 @@ thunar_chooser_dialog_class_init (ThunarChooserDialogClass *klass)
   GtkWidgetClass *gtkwidget_class;
   GObjectClass   *gobject_class;
 
-  /* determine the parent type class */
-  thunar_chooser_dialog_parent_class = g_type_class_peek_parent (klass);
-
   gobject_class = G_OBJECT_CLASS (klass);
   gobject_class->dispose = thunar_chooser_dialog_dispose;
   gobject_class->get_property = thunar_chooser_dialog_get_property;
@@ -214,7 +184,6 @@ thunar_chooser_dialog_init (ThunarChooserDialog *dialog)
   GtkWidget         *swin;
 
   /* setup basic window properties */
-  gtk_dialog_set_has_separator (GTK_DIALOG (dialog), FALSE);
   gtk_window_set_destroy_with_parent (GTK_WINDOW (dialog), TRUE);
   gtk_window_set_resizable (GTK_WINDOW (dialog), FALSE);
   gtk_window_set_title (GTK_WINDOW (dialog), _("Open With"));
@@ -268,7 +237,7 @@ thunar_chooser_dialog_init (ThunarChooserDialog *dialog)
   renderer = g_object_new (EXO_TYPE_CELL_RENDERER_ICON, "follow-state", FALSE, "size", 24, NULL);
   gtk_tree_view_column_pack_start (column, renderer, FALSE);
   gtk_tree_view_column_set_attributes (column, renderer,
-                                       "icon", THUNAR_CHOOSER_MODEL_COLUMN_ICON,
+                                       "gicon", THUNAR_CHOOSER_MODEL_COLUMN_ICON,
                                        NULL);
   renderer = gtk_cell_renderer_text_new ();
   gtk_tree_view_column_pack_start (column, renderer, TRUE);
@@ -279,21 +248,18 @@ thunar_chooser_dialog_init (ThunarChooserDialog *dialog)
                                        "weight", THUNAR_CHOOSER_MODEL_COLUMN_WEIGHT,
                                        "weight-set", THUNAR_CHOOSER_MODEL_COLUMN_WEIGHT_SET,
                                        NULL);
-  gtk_tree_view_column_set_sort_column_id (column, THUNAR_CHOOSER_MODEL_COLUMN_NAME);
   gtk_tree_view_append_column (GTK_TREE_VIEW (dialog->tree_view), column);
 
-#if GTK_CHECK_VERSION(2,9,0)
-  /* don't show the expanders with GTK+ 2.9 and above */
+  /* don't show the expanders */
   g_object_set (G_OBJECT (dialog->tree_view),
                 "level-indentation", 24,
                 "show-expanders", FALSE,
                 NULL);
-#endif
 
   /* create the "Custom command" expand */
   dialog->custom_expander = gtk_expander_new_with_mnemonic (_("Use a _custom command:"));
-  thunar_gtk_widget_set_tooltip (dialog->custom_expander, _("Use a custom command for an application that is not "
-                                                            "available from the above application list."));
+  gtk_widget_set_tooltip_text (dialog->custom_expander, _("Use a custom command for an application that is not "
+                                                          "available from the above application list."));
   exo_binding_new_with_negation (G_OBJECT (dialog->custom_expander), "expanded", G_OBJECT (dialog->tree_view), "sensitive");
   g_signal_connect (G_OBJECT (dialog->custom_expander), "notify::expanded", G_CALLBACK (thunar_chooser_dialog_notify_expanded), dialog);
   gtk_box_pack_start (GTK_BOX (box), dialog->custom_expander, FALSE, FALSE, 0);
@@ -408,23 +374,12 @@ static void
 thunar_chooser_dialog_realize (GtkWidget *widget)
 {
   ThunarChooserDialog *dialog = THUNAR_CHOOSER_DIALOG (widget);
-  GtkTreeModel        *model;
-  GdkCursor           *cursor;
 
   /* let the GtkWindow class realize the dialog */
   (*GTK_WIDGET_CLASS (thunar_chooser_dialog_parent_class)->realize) (widget);
 
   /* update the dialog header */
   thunar_chooser_dialog_update_header (dialog);
-
-  /* setup a watch cursor if we're currently loading the model */
-  model = gtk_tree_view_get_model (GTK_TREE_VIEW (dialog->tree_view));
-  if (thunar_chooser_model_get_loading (THUNAR_CHOOSER_MODEL (model)))
-    {
-      cursor = gdk_cursor_new (GDK_WATCH);
-      gdk_window_set_cursor (widget->window, cursor);
-      gdk_cursor_unref (cursor);
-    }
 }
 
 
@@ -433,37 +388,34 @@ static void
 thunar_chooser_dialog_response (GtkDialog *widget,
                                 gint       response)
 {
-  ThunarVfsMimeApplication *application = NULL;
-  ThunarVfsMimeDatabase    *mime_database;
-  ThunarChooserDialog      *dialog = THUNAR_CHOOSER_DIALOG (widget);
-  ThunarVfsMimeInfo        *mime_info;
-  GtkTreeSelection         *selection;
-  GtkTreeModel             *model;
-  GtkTreeIter               iter;
-  const gchar              *exec;
-  gboolean                  succeed = TRUE;
-  GError                   *error = NULL;
-  gchar                    *path;
-  gchar                    *name;
-  gchar                    *s;
-  GList                     list;
+  GdkAppLaunchContext *context;
+  ThunarChooserDialog *dialog = THUNAR_CHOOSER_DIALOG (widget);
+  GtkTreeSelection    *selection;
+  GtkTreeModel        *model;
+  GtkTreeIter          iter;
+  const gchar         *content_type;
+  const gchar         *exec;
+  GAppInfo            *app_info;
+  gboolean             succeed = TRUE;
+  GError              *error = NULL;
+  gchar               *path;
+  gchar               *name;
+  gchar               *s;
+  GList                list;
 
   /* no special processing for non-accept responses */
   if (G_UNLIKELY (response != GTK_RESPONSE_ACCEPT))
     return;
 
-  /* grab a reference on the mime database */
-  mime_database = thunar_vfs_mime_database_get_default ();
-
-  /* determine the mime info for the file */
-  mime_info = thunar_file_get_mime_info (dialog->file);
+  /* determine the content type for the file */
+  content_type = thunar_file_get_content_type (dialog->file);
 
   /* determine the application that was chosen by the user */
   if (!gtk_expander_get_expanded (GTK_EXPANDER (dialog->custom_expander)))
     {
       selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (dialog->tree_view));
       if (gtk_tree_selection_get_selected (selection, &model, &iter))
-        gtk_tree_model_get (model, &iter, THUNAR_CHOOSER_MODEL_COLUMN_APPLICATION, &application, -1);
+        gtk_tree_model_get (model, &iter, THUNAR_CHOOSER_MODEL_COLUMN_APPLICATION, &app_info, -1);
     }
   else
     {
@@ -480,10 +432,10 @@ thunar_chooser_dialog_response (GtkDialog *widget,
       name = g_path_get_basename (path);
 
       /* try to add an application for the custom command */
-      application = thunar_vfs_mime_database_add_application (mime_database, mime_info, name, exec, &error);
+      app_info = g_app_info_create_from_commandline (exec, name, G_APP_INFO_CREATE_NONE, &error);
 
       /* verify the application */
-      if (G_UNLIKELY (application == NULL))
+      if (G_UNLIKELY (app_info == NULL))
         {
           /* display an error to the user */
           thunar_dialogs_show_error (GTK_WIDGET (dialog), error, _("Failed to add new application \"%s\""), name);
@@ -498,20 +450,22 @@ thunar_chooser_dialog_response (GtkDialog *widget,
     }
 
   /* verify that we have a valid application */
-  if (G_UNLIKELY (application == NULL))
-    goto cleanup;
+  if (G_UNLIKELY (app_info == NULL))
+    return;
 
   /* check if we should also set the application as default */
   if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (dialog->default_button)))
     {
       /* remember the application as default for these kind of file */
-      succeed = thunar_vfs_mime_database_set_default_application (mime_database, mime_info, application, &error);
+      succeed = g_app_info_set_as_default_for_type (app_info, content_type, &error);
 
       /* verify that we were successfull */
       if (G_UNLIKELY (!succeed))
         {
           /* display an error to the user */
-          thunar_dialogs_show_error (GTK_WIDGET (dialog), error, _("Failed to set default application for \"%s\""),
+          thunar_dialogs_show_error (GTK_WIDGET (dialog), 
+                                     error, 
+                                     _("Failed to set default application for \"%s\""),
                                      thunar_file_get_display_name (dialog->file));
 
           /* release the error */
@@ -526,23 +480,32 @@ thunar_chooser_dialog_response (GtkDialog *widget,
   /* check if we should also execute the application */
   if (G_LIKELY (succeed && dialog->open))
     {
-      /* open the file using the specified application */
-      list.data = thunar_file_get_path (dialog->file); list.next = list.prev = NULL;
-      if (!thunar_vfs_mime_handler_exec (THUNAR_VFS_MIME_HANDLER (application), gtk_widget_get_screen (GTK_WIDGET (dialog)), &list, &error))
+      /* create launch context */
+      context = gdk_app_launch_context_new ();
+      gdk_app_launch_context_set_screen (context, gtk_widget_get_screen (GTK_WIDGET (dialog)));
+      gdk_app_launch_context_set_timestamp (context, gtk_get_current_event_time ());
+
+      /* create fake file list */
+      list.data = thunar_file_get_file (dialog->file); list.next = list.prev = NULL;
+
+      if (!g_app_info_launch (app_info, &list, G_APP_LAUNCH_CONTEXT (context), &error))
         {
           /* display an error to the user */
-          thunar_dialogs_show_error (GTK_WIDGET (dialog), error, _("Failed to execute \"%s\""),
-                                     thunar_vfs_mime_handler_get_name (THUNAR_VFS_MIME_HANDLER (application)));
+          thunar_dialogs_show_error (GTK_WIDGET (dialog), 
+                                     error, 
+                                     _("Failed to execute application \"%s\""), 
+                                     g_app_info_get_name (app_info));
 
           /* release the error */
           g_error_free (error);
         }
+
+      /* destroy the launch context */
+      g_object_unref (context);
     }
 
   /* cleanup */
-  g_object_unref (G_OBJECT (application));
-cleanup:
-  g_object_unref (G_OBJECT (mime_database));
+  g_object_unref (app_info);
 }
 
 
@@ -576,15 +539,15 @@ thunar_chooser_dialog_selection_func (GtkTreeSelection *selection,
 static gboolean
 thunar_chooser_dialog_context_menu (ThunarChooserDialog *dialog,
                                     guint                button,
-                                    guint32              time)
+                                    guint32              timestamp)
 {
-  ThunarVfsMimeApplication *mime_application;
-  GtkTreeSelection         *selection;
-  GtkTreeModel             *model;
-  GtkTreeIter               iter;
-  GtkWidget                *image;
-  GtkWidget                *item;
-  GtkWidget                *menu;
+  GtkTreeSelection *selection;
+  GtkTreeModel     *model;
+  GtkTreeIter       iter;
+  GtkWidget        *image;
+  GtkWidget        *item;
+  GtkWidget        *menu;
+  GAppInfo         *app_info;
 
   _thunar_return_val_if_fail (THUNAR_IS_CHOOSER_DIALOG (dialog), FALSE);
 
@@ -593,9 +556,9 @@ thunar_chooser_dialog_context_menu (ThunarChooserDialog *dialog,
   if (!gtk_tree_selection_get_selected (selection, &model, &iter))
     return FALSE;
 
-  /* determine the mime application for the row */
-  gtk_tree_model_get (model, &iter, THUNAR_CHOOSER_MODEL_COLUMN_APPLICATION, &mime_application, -1);
-  if (G_UNLIKELY (mime_application == NULL))
+  /* determine the app info for the row */
+  gtk_tree_model_get (model, &iter, THUNAR_CHOOSER_MODEL_COLUMN_APPLICATION, &app_info, -1);
+  if (G_UNLIKELY (app_info == NULL))
     return FALSE;
 
   /* prepare the popup menu */
@@ -603,20 +566,19 @@ thunar_chooser_dialog_context_menu (ThunarChooserDialog *dialog,
 
   /* append the "Remove Launcher" item */
   item = gtk_image_menu_item_new_with_mnemonic (_("_Remove Launcher"));
-  gtk_widget_set_sensitive (item, thunar_vfs_mime_application_is_usercreated (mime_application));
+  gtk_widget_set_sensitive (item, g_app_info_can_delete (app_info));
   g_signal_connect_swapped (G_OBJECT (item), "activate", G_CALLBACK (thunar_chooser_dialog_action_remove), dialog);
   gtk_menu_shell_append (GTK_MENU_SHELL (menu), item);
   gtk_widget_show (item);
 
   image = gtk_image_new_from_stock (GTK_STOCK_REMOVE, GTK_ICON_SIZE_MENU);
   gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (item), image);
-  gtk_widget_show (image);
 
   /* run the menu on the dialog's screen (takes over the floating of menu) */
-  thunar_gtk_menu_run (GTK_MENU (menu), GTK_WIDGET (dialog), NULL, NULL, button, time);
+  thunar_gtk_menu_run (GTK_MENU (menu), GTK_WIDGET (dialog), NULL, NULL, button, timestamp);
 
   /* clean up */
-  g_object_unref (G_OBJECT (mime_application));
+  g_object_unref (app_info);
 
   return TRUE;
 }
@@ -662,45 +624,33 @@ thunar_chooser_dialog_update_accept (ThunarChooserDialog *dialog)
 static void
 thunar_chooser_dialog_update_header (ThunarChooserDialog *dialog)
 {
-  ThunarVfsMimeInfo *mime_info;
-  ThunarIconFactory *icon_factory;
-  GtkIconTheme      *icon_theme;
-  const gchar       *icon_name;
-  GdkPixbuf         *icon;
-  gchar             *text;
+  const gchar *content_type;
+  GIcon       *icon;
+  gchar       *description;
+  gchar       *text;
 
   _thunar_return_if_fail (THUNAR_IS_CHOOSER_DIALOG (dialog));
-  _thunar_return_if_fail (GTK_WIDGET_REALIZED (dialog));
+  _thunar_return_if_fail (gtk_widget_get_realized (GTK_WIDGET (dialog)));
 
   /* check if we have a valid file set */
   if (G_UNLIKELY (dialog->file == NULL))
     {
-#if GTK_CHECK_VERSION(2,8,0)
       gtk_image_clear (GTK_IMAGE (dialog->header_image));
-#endif
       gtk_label_set_text (GTK_LABEL (dialog->header_label), NULL);
     }
   else
     {
-      /* determine the mime info for the file */
-      mime_info = thunar_file_get_mime_info (dialog->file);
+      content_type = thunar_file_get_content_type (dialog->file);
+      description = g_content_type_get_description (content_type);
 
-      /* determine the icon theme/factory for the widget */
-      icon_theme = gtk_icon_theme_get_for_screen (gtk_widget_get_screen (GTK_WIDGET (dialog)));
-      icon_factory = thunar_icon_factory_get_for_icon_theme (icon_theme);
-
-      /* update the header image with the icon for the mime type */
-      icon_name = thunar_vfs_mime_info_lookup_icon_name (mime_info, icon_theme);
-      icon = thunar_icon_factory_load_icon (icon_factory, icon_name, 48, NULL, FALSE);
-      gtk_image_set_from_pixbuf (GTK_IMAGE (dialog->header_image), icon);
-      gtk_window_set_icon (GTK_WINDOW (dialog), icon);
-      if (G_LIKELY (icon != NULL))
-        g_object_unref (G_OBJECT (icon));
+      icon = g_content_type_get_icon (content_type);
+      gtk_image_set_from_gicon (GTK_IMAGE (dialog->header_image), icon, GTK_ICON_SIZE_DIALOG);
+      g_object_unref (icon);
 
       /* update the header label */
       text = g_strdup_printf (_("Open <i>%s</i> and other files of type \"%s\" with:"),
                               thunar_file_get_display_name (dialog->file),
-                              thunar_vfs_mime_info_get_comment (mime_info));
+                              description);
       gtk_label_set_markup (GTK_LABEL (dialog->header_label), text);
       g_free (text);
 
@@ -708,16 +658,16 @@ thunar_chooser_dialog_update_header (ThunarChooserDialog *dialog)
       thunar_gtk_widget_set_tooltip (dialog->custom_button,
                                      _("Browse the file system to select an "
                                        "application to open files of type \"%s\"."),
-                                     thunar_vfs_mime_info_get_comment (mime_info));
+                                     description);
 
       /* update the "Use as default for this kind of file" tooltip */
       thunar_gtk_widget_set_tooltip (dialog->default_button,
                                      _("Change the default application for files "
                                        "of type \"%s\" to the selected application."),
-                                     thunar_vfs_mime_info_get_comment (mime_info));
+                                     description);
 
       /* cleanup */
-      g_object_unref (G_OBJECT (icon_factory));
+      g_free (description);
     }
 }
 
@@ -726,14 +676,14 @@ thunar_chooser_dialog_update_header (ThunarChooserDialog *dialog)
 static void
 thunar_chooser_dialog_action_remove (ThunarChooserDialog *dialog)
 {
-  ThunarVfsMimeApplication *mime_application;
-  GtkTreeSelection         *selection;
-  GtkTreeModel             *model;
-  GtkTreeIter               iter;
-  const gchar              *name;
-  GtkWidget                *message;
-  GError                   *error = NULL;
-  gint                      response;
+  GtkTreeSelection *selection;
+  GtkTreeModel     *model;
+  GtkTreeIter       iter;
+  const gchar      *name;
+  GtkWidget        *message;
+  GAppInfo         *app_info;
+  GError           *error = NULL;
+  gint              response;
 
   _thunar_return_if_fail (THUNAR_IS_CHOOSER_DIALOG (dialog));
 
@@ -742,16 +692,15 @@ thunar_chooser_dialog_action_remove (ThunarChooserDialog *dialog)
   if (!gtk_tree_selection_get_selected (selection, &model, &iter))
     return;
 
-  /* determine the mime application for the row */
-  gtk_tree_model_get (model, &iter, THUNAR_CHOOSER_MODEL_COLUMN_APPLICATION, &mime_application, -1);
-  if (G_UNLIKELY (mime_application == NULL))
+  /* determine the app info for the row */
+  gtk_tree_model_get (model, &iter, THUNAR_CHOOSER_MODEL_COLUMN_APPLICATION, &app_info, -1);
+  if (G_UNLIKELY (app_info == NULL))
     return;
 
-  /* verify that the application is usercreated */
-  if (thunar_vfs_mime_application_is_usercreated (mime_application))
+  if (g_app_info_can_delete (app_info))
     {
-      /* determine the name of the mime application */
-      name = thunar_vfs_mime_handler_get_name (THUNAR_VFS_MIME_HANDLER (mime_application));
+      /* determine the name of the app info */
+      name = g_app_info_get_name (app_info);
 
       /* ask the user whether to remove the application launcher */
       message = gtk_message_dialog_new (GTK_WINDOW (dialog),
@@ -792,7 +741,7 @@ thunar_chooser_dialog_action_remove (ThunarChooserDialog *dialog)
     }
 
   /* cleanup */
-  g_object_unref (G_OBJECT (mime_application));
+  g_object_unref (app_info);
 }
 
 
@@ -970,16 +919,16 @@ thunar_chooser_dialog_notify_expanded (GtkExpander         *expander,
 
 
 static void
-thunar_chooser_dialog_notify_loading (ThunarChooserModel  *model,
-                                      GParamSpec          *pspec,
-                                      ThunarChooserDialog *dialog)
+thunar_chooser_dialog_expand (ThunarChooserDialog *dialog)
 {
-  GtkTreePath *path;
-  GtkTreeIter  iter;
+  GtkTreeModel *model;
+  GtkTreePath  *path;
+  GtkTreeIter   iter;
 
-  _thunar_return_if_fail (THUNAR_IS_CHOOSER_MODEL (model));
   _thunar_return_if_fail (THUNAR_IS_CHOOSER_DIALOG (dialog));
 
+  model = gtk_tree_view_get_model (GTK_TREE_VIEW (dialog->tree_view));
+
   /* expand the first tree view row (the recommended applications) */
   if (G_LIKELY (gtk_tree_model_get_iter_first (GTK_TREE_MODEL (model), &iter)))
     {
@@ -988,7 +937,6 @@ thunar_chooser_dialog_notify_loading (ThunarChooserModel  *model,
       gtk_tree_path_free (path);
     }
 
-#if GTK_CHECK_VERSION(2,9,0)
   /* expand the second tree view row (the other applications) */
   if (G_LIKELY (gtk_tree_model_iter_next (GTK_TREE_MODEL (model), &iter)))
     {
@@ -996,14 +944,13 @@ thunar_chooser_dialog_notify_loading (ThunarChooserModel  *model,
       gtk_tree_view_expand_to_path (GTK_TREE_VIEW (dialog->tree_view), path);
       gtk_tree_path_free (path);
     }
-#endif
 
   /* reset the cursor */
-  if (G_LIKELY (GTK_WIDGET_REALIZED (dialog)))
+  if (G_LIKELY (gtk_widget_get_realized (GTK_WIDGET (dialog))))
     gdk_window_set_cursor (GTK_WIDGET (dialog)->window, NULL);
 
   /* grab focus to the tree view widget */
-  if (G_LIKELY (GTK_WIDGET_REALIZED (dialog->tree_view)))
+  if (G_LIKELY (gtk_widget_get_realized (dialog->tree_view)))
     gtk_widget_grab_focus (dialog->tree_view);
 }
 
@@ -1072,19 +1019,19 @@ static void
 thunar_chooser_dialog_selection_changed (GtkTreeSelection    *selection,
                                          ThunarChooserDialog *dialog)
 {
-  ThunarVfsMimeApplication *mime_application;
-  GtkTreeModel             *model;
-  const gchar              *exec;
-  GtkTreeIter               iter;
+  GAppInfo     *app_info;
+  GtkTreeModel *model;
+  const gchar  *exec;
+  GtkTreeIter   iter;
 
   /* determine the iterator for the selected row */
   if (gtk_tree_selection_get_selected (selection, &model, &iter))
     {
-      /* determine the mime application for the selected row */
-      gtk_tree_model_get (model, &iter, THUNAR_CHOOSER_MODEL_COLUMN_APPLICATION, &mime_application, -1);
+      /* determine the app info for the selected row */
+      gtk_tree_model_get (model, &iter, THUNAR_CHOOSER_MODEL_COLUMN_APPLICATION, &app_info, -1);
 
-      /* determine the command for the mime application */
-      exec = thunar_vfs_mime_handler_get_command (THUNAR_VFS_MIME_HANDLER (mime_application));
+      /* determine the command for the app info */
+      exec = g_app_info_get_executable (app_info);
       if (G_LIKELY (exec != NULL && g_utf8_validate (exec, -1, NULL)))
         {
           /* setup the command as default for the custom command box */
@@ -1092,7 +1039,7 @@ thunar_chooser_dialog_selection_changed (GtkTreeSelection    *selection,
         }
 
       /* cleanup */
-      g_object_unref (G_OBJECT (mime_application));
+      g_object_unref (app_info);
     }
 
   /* update the sensitivity of the "Ok"/"Open" button */
@@ -1101,21 +1048,6 @@ thunar_chooser_dialog_selection_changed (GtkTreeSelection    *selection,
 
 
 
-/**
- * thunar_chooser_dialog_new:
- *
- * Allocates a new #ThunarChooserDialog.
- *
- * Return value: the newly allocated #ThunarChooserDialog.
- **/
-GtkWidget*
-thunar_chooser_dialog_new (void)
-{
-  return g_object_new (THUNAR_TYPE_CHOOSER_DIALOG, NULL);
-}
-
-
-
 /**
  * thunar_chooser_dialog_get_file:
  * @dialog : a #ThunarChooserDialog.
@@ -1125,7 +1057,7 @@ thunar_chooser_dialog_new (void)
  *
  * Return value: the #ThunarFile associated with @dialog.
  **/
-ThunarFile*
+static ThunarFile*
 thunar_chooser_dialog_get_file (ThunarChooserDialog *dialog)
 {
   _thunar_return_val_if_fail (THUNAR_IS_CHOOSER_DIALOG (dialog), NULL);
@@ -1141,12 +1073,11 @@ thunar_chooser_dialog_get_file (ThunarChooserDialog *dialog)
  *
  * Associates @dialog with @file.
  **/
-void
+static void
 thunar_chooser_dialog_set_file (ThunarChooserDialog *dialog,
                                 ThunarFile          *file)
 {
   ThunarChooserModel *model;
-  ThunarVfsMimeInfo  *mime_info;
 
   _thunar_return_if_fail (THUNAR_IS_CHOOSER_DIALOG (dialog));
   _thunar_return_if_fail (file == NULL || THUNAR_IS_FILE (file));
@@ -1179,15 +1110,14 @@ thunar_chooser_dialog_set_file (ThunarChooserDialog *dialog,
       g_signal_connect_swapped (G_OBJECT (file), "destroy", G_CALLBACK (gtk_widget_destroy), dialog);
 
       /* allocate the new chooser model */
-      mime_info = thunar_file_get_mime_info (file);
-      model = thunar_chooser_model_new (mime_info);
+      model = thunar_chooser_model_new (thunar_file_get_content_type (file));
       gtk_tree_view_set_model (GTK_TREE_VIEW (dialog->tree_view), GTK_TREE_MODEL (model));
-      g_signal_connect (G_OBJECT (model), "notify::loading", G_CALLBACK (thunar_chooser_dialog_notify_loading), dialog);
+      thunar_chooser_dialog_expand (dialog);
       g_object_unref (G_OBJECT (model));
     }
 
   /* update the header */
-  if (GTK_WIDGET_REALIZED (dialog))
+  if (gtk_widget_get_realized (GTK_WIDGET (dialog)))
     thunar_chooser_dialog_update_header (dialog);
 
   /* notify listeners */
@@ -1204,7 +1134,7 @@ thunar_chooser_dialog_set_file (ThunarChooserDialog *dialog,
  *
  * Return value: %TRUE if @dialog should open the file.
  **/
-gboolean
+static gboolean
 thunar_chooser_dialog_get_open (ThunarChooserDialog *dialog)
 {
   _thunar_return_val_if_fail (THUNAR_IS_CHOOSER_DIALOG (dialog), FALSE);
@@ -1220,7 +1150,7 @@ thunar_chooser_dialog_get_open (ThunarChooserDialog *dialog)
  *
  * Sets whether the chooser @dialog should open the file.
  **/
-void
+static void
 thunar_chooser_dialog_set_open (ThunarChooserDialog *dialog,
                                 gboolean             open)
 {
@@ -1293,7 +1223,7 @@ thunar_show_chooser_dialog (gpointer    parent,
                          NULL);
 
   /* check if we have a toplevel window */
-  if (G_LIKELY (window != NULL && GTK_WIDGET_TOPLEVEL (window)))
+  if (G_LIKELY (window != NULL && gtk_widget_get_toplevel (window)))
     {
       /* dialog is transient for toplevel window and modal */
       gtk_window_set_destroy_with_parent (GTK_WINDOW (dialog), TRUE);
