@@ -207,6 +207,7 @@ thunar_chooser_model_append (ThunarChooserModel *model,
   GtkTreeIter parent_iter;
   GIcon      *icon;
   GList      *lp;
+  gboolean    inserted_infos = FALSE;
 
   _thunar_return_if_fail (THUNAR_IS_CHOOSER_MODEL (model));
   _thunar_return_if_fail (title != NULL);
@@ -229,6 +230,10 @@ thunar_chooser_model_append (ThunarChooserModel *model,
       /* insert the program items */
       for (lp = app_infos; lp != NULL; lp = lp->next)
         {
+          /* skip infos that have NoDisplay or OnlyShowIn set */
+          if (!g_app_info_should_show (lp->data))
+            continue;
+
           /* append the tree row with the program data */
           gtk_tree_store_append (GTK_TREE_STORE (model), &child_iter, &parent_iter);
           gtk_tree_store_set (GTK_TREE_STORE (model), &child_iter,
@@ -236,9 +241,12 @@ thunar_chooser_model_append (ThunarChooserModel *model,
                               THUNAR_CHOOSER_MODEL_COLUMN_ICON, g_app_info_get_icon (lp->data),
                               THUNAR_CHOOSER_MODEL_COLUMN_APPLICATION, lp->data,
                               -1);
+
+          inserted_infos = TRUE;
         }
     }
-  else
+  
+  if (!inserted_infos)
     {
       /* tell the user that we don't have any applications for this category */
       gtk_tree_store_append (GTK_TREE_STORE (model), &child_iter, &parent_iter);
@@ -288,6 +296,7 @@ thunar_chooser_model_reload (ThunarChooserModel *model)
   recommended = g_app_info_get_all_for_type (model->content_type);
 
   /* append them as recommended */
+  recommended = g_list_sort (recommended, sort_app_infos);
   thunar_chooser_model_append (model, 
                                _("Recommended Applications"), 
                                "preferences-desktop-default-applications", 
@@ -311,11 +320,8 @@ thunar_chooser_model_reload (ThunarChooserModel *model)
                                "gnome-applications",
                                other);
 
-  g_list_foreach (recommended, (GFunc) g_object_unref, NULL);
-  g_list_free (recommended);
-
-  g_list_foreach (all, (GFunc) g_object_unref, NULL);
-  g_list_free (all);
+  g_list_free_full (recommended, g_object_unref);
+  g_list_free_full (all, g_object_unref);
 
   g_list_free (other);
 }
