@@ -103,6 +103,13 @@ static gboolean thunar_dbus_service_launch                      (ThunarDBusServi
                                                                  const gchar            *display,
                                                                  const gchar            *startup_id,
                                                                  GError                **error);
+static gboolean thunar_dbus_service_execute                     (ThunarDBusService      *dbus_service,
+                                                                 const gchar            *working_directory,
+                                                                 const gchar            *uri,
+                                                                 const gchar           **files,
+                                                                 const gchar            *display,
+                                                                 const gchar            *startup_id,
+                                                                 GError                **error);
 static gboolean thunar_dbus_service_display_preferences_dialog  (ThunarDBusService      *dbus_service,
                                                                  const gchar            *display,
                                                                  const gchar            *startup_id,
@@ -136,6 +143,23 @@ static gboolean thunar_dbus_service_launch_files                (ThunarDBusServi
                                                                  const gchar            *display,
                                                                  const gchar            *startup_id,
                                                                  GError                **error);
+static gboolean thunar_dbus_service_rename_file                 (ThunarDBusService      *dbus_service,
+                                                                 const gchar            *uri,
+                                                                 const gchar            *display,
+                                                                 const gchar            *startup_id,
+                                                                 GError                **error);
+static gboolean thunar_dbus_service_create_file                 (ThunarDBusService      *dbus_service,
+                                                                 const gchar            *parent_directory,
+                                                                 const gchar            *content_type,
+                                                                 const gchar            *display,
+                                                                 const gchar            *startup_id,
+                                                                 GError                **error);
+static gboolean thunar_dbus_service_create_file_from_template   (ThunarDBusService      *dbus_service,
+                                                                 const gchar            *parent_directory,
+                                                                 const gchar            *template_uri,
+                                                                 const gchar            *display,
+                                                                 const gchar            *startup_id,
+                                                                 GError                **error);
 static gboolean thunar_dbus_service_copy_to                     (ThunarDBusService      *dbus_service,
                                                                  const gchar            *working_directory,
                                                                  gchar                 **source_filenames,
@@ -234,7 +258,9 @@ thunar_dbus_service_class_init (ThunarDBusServiceClass *klass)
 static void
 thunar_dbus_service_init (ThunarDBusService *dbus_service)
 {
-  GError *error = NULL;
+  GError         *error = NULL;
+  DBusConnection *dbus_connection;
+  gint            result;
 
   /* try to connect to the session bus */
   dbus_service->connection = dbus_g_bus_get (DBUS_BUS_SESSION, &error);
@@ -244,15 +270,33 @@ thunar_dbus_service_init (ThunarDBusService *dbus_service)
       dbus_g_connection_register_g_object (dbus_service->connection, "/org/xfce/FileManager", G_OBJECT (dbus_service));
 
       /* request the org.xfce.Thunar name for Thunar */
-      dbus_bus_request_name (dbus_g_connection_get_connection (dbus_service->connection), "org.xfce.Thunar", DBUS_NAME_FLAG_REPLACE_EXISTING, NULL);
+      dbus_connection = dbus_g_connection_get_connection (dbus_service->connection);
+      result = dbus_bus_request_name (dbus_connection, "org.xfce.Thunar",
+                                      DBUS_NAME_FLAG_ALLOW_REPLACEMENT | DBUS_NAME_FLAG_DO_NOT_QUEUE, NULL);
+
+      /* check if we successfully acquired the name */
+      if (result != DBUS_REQUEST_NAME_REPLY_PRIMARY_OWNER)
+        {
+          g_printerr ("Thunar: D-BUS name org.xfce.Thunar already registered.\n");
+
+          /* unset connection */
+          dbus_g_connection_unref (dbus_service->connection);
+          dbus_service->connection = NULL;
+
+          return;
+        }
 
       /* request the org.xfce.FileManager name for Thunar */
-      dbus_bus_request_name (dbus_g_connection_get_connection (dbus_service->connection), "org.xfce.FileManager", DBUS_NAME_FLAG_REPLACE_EXISTING, NULL);
+      dbus_bus_request_name (dbus_connection, "org.xfce.FileManager", DBUS_NAME_FLAG_REPLACE_EXISTING, NULL);
+
+      /* once we registered, unset dbus variables (bug #8800) */
+      g_unsetenv ("DBUS_STARTER_ADDRESS");
+      g_unsetenv ("DBUS_STARTER_BUS_TYPE");
     }
   else
     {
       /* notify the user that D-BUS service won't be available */
-      g_fprintf (stderr, "Thunar: Failed to connect to the D-BUS session bus: %s\n", error->message);
+      g_printerr ("Thunar: Failed to connect to the D-BUS session bus: %s\n", error->message);
       g_error_free (error);
     }
 }
@@ -263,10 +307,18 @@ static void
 thunar_dbus_service_finalize (GObject *object)
 {
   ThunarDBusService *dbus_service = THUNAR_DBUS_SERVICE (object);
+  DBusConnection    *dbus_connection;
 
   /* release the D-BUS connection object */
   if (G_LIKELY (dbus_service->connection != NULL))
-    dbus_g_connection_unref (dbus_service->connection);
+    {
+      /* release the names */
+      dbus_connection = dbus_g_connection_get_connection (dbus_service->connection);
+      dbus_bus_release_name (dbus_connection, "org.xfce.Thunar", NULL);
+      dbus_bus_release_name (dbus_connection, "org.xfce.FileManager", NULL);
+
+      dbus_g_connection_unref (dbus_service->connection);
+    }
 
   /* check if we are connected to the trash bin */
   if (G_LIKELY (dbus_service->trash_bin != NULL))
@@ -488,7 +540,7 @@ thunar_dbus_service_display_file_properties (ThunarDBusService *dbus_service,
     return FALSE;
 
   /* popup the file properties dialog */
-  dialog = thunar_properties_dialog_new ();
+  dialog = thunar_properties_dialog_new (NULL);
   gtk_window_set_screen (GTK_WINDOW (dialog), screen);
   gtk_window_set_startup_id (GTK_WINDOW (dialog), startup_id);
   thunar_properties_dialog_set_file (THUNAR_PROPERTIES_DIALOG (dialog), file);
@@ -535,6 +587,58 @@ thunar_dbus_service_launch (ThunarDBusService *dbus_service,
 
 
 
+static gboolean
+thunar_dbus_service_execute (ThunarDBusService *dbus_service,
+                             const gchar       *working_directory,
+                             const gchar       *uri,
+                             const gchar      **files,
+                             const gchar       *display,
+                             const gchar       *startup_id,
+                             GError           **error)
+{
+  ThunarFile *file;
+  GdkScreen  *screen;
+  gboolean    result = FALSE;
+  GFile      *working_dir;
+  GList      *file_list = NULL;
+  gchar      *tmp_working_dir = NULL;
+  gchar      *old_working_dir = NULL;
+  guint       n;
+
+  /* parse uri and display parameters */
+  if (thunar_dbus_service_parse_uri_and_display (dbus_service, uri, display, &file, &screen, error))
+    {
+      if (working_directory != NULL && *working_directory != '\0')
+        old_working_dir = thunar_util_change_working_directory (working_directory);
+
+      for (n = 0; files != NULL && files[n] != NULL; ++n)
+        file_list = g_list_prepend (file_list, g_file_new_for_commandline_arg (files[n]));
+
+      file_list = g_list_reverse (file_list);
+
+      if (old_working_dir != NULL)
+        {
+          tmp_working_dir = thunar_util_change_working_directory (old_working_dir);
+          g_free (tmp_working_dir);
+          g_free (old_working_dir);
+        }
+
+      /* try to launch the file on the given screen */
+      working_dir = g_file_new_for_commandline_arg (working_directory);
+      result = thunar_file_execute (file, working_dir, screen, file_list, error);
+      g_object_unref (working_dir);
+
+      /* cleanup */
+      g_list_free_full (file_list, g_object_unref);
+      g_object_unref (screen);
+      g_object_unref (file);
+    }
+
+  return result;
+}
+
+
+
 static gboolean
 thunar_dbus_service_display_preferences_dialog (ThunarDBusService *dbus_service,
                                                 const gchar       *display,
@@ -797,6 +901,105 @@ thunar_dbus_service_launch_files (ThunarDBusService *dbus_service,
 
 
 
+static gboolean
+thunar_dbus_service_rename_file (ThunarDBusService *dbus_service,
+                                 const gchar       *uri,
+                                 const gchar       *display,
+                                 const gchar       *startup_id,
+                                 GError           **error)
+{
+  ThunarApplication *application;
+  ThunarFile        *file;
+  GdkScreen         *screen;
+
+  /* parse uri and display parameters */
+  if (!thunar_dbus_service_parse_uri_and_display (dbus_service, uri, display, &file, &screen, error))
+    return FALSE;
+
+  /* popup a new window for the folder */
+  application = thunar_application_get ();
+  thunar_application_rename_file (application, file, screen, startup_id);
+  g_object_unref (G_OBJECT (application));
+
+  /* cleanup */
+  g_object_unref (G_OBJECT (screen));
+  g_object_unref (G_OBJECT (file));
+
+  return TRUE;
+}
+
+
+
+static gboolean
+thunar_dbus_service_create_file (ThunarDBusService *dbus_service,
+                                 const gchar       *parent_directory,
+                                 const gchar       *content_type,
+                                 const gchar       *display,
+                                 const gchar       *startup_id,
+                                 GError           **error)
+{
+  ThunarApplication *application;
+  ThunarFile        *file;
+  GdkScreen         *screen;
+
+  /* parse uri and display parameters */
+  if (!thunar_dbus_service_parse_uri_and_display (dbus_service, parent_directory, display, &file, &screen, error))
+    return FALSE;
+
+  /* fall back to plain text file if no content type is provided */
+  if (content_type == NULL || *content_type == '\0')
+    content_type = "text/plain";
+
+  /* popup a new window for the folder */
+  application = thunar_application_get ();
+  thunar_application_create_file (application, file, content_type, screen, startup_id);
+  g_object_unref (G_OBJECT (application));
+
+  /* cleanup */
+  g_object_unref (G_OBJECT (screen));
+  g_object_unref (G_OBJECT (file));
+
+  return TRUE;
+}
+
+
+
+static gboolean
+thunar_dbus_service_create_file_from_template (ThunarDBusService *dbus_service,
+                                               const gchar       *parent_directory,
+                                               const gchar       *template_uri,
+                                               const gchar       *display,
+                                               const gchar       *startup_id,
+                                               GError           **error)
+{
+  ThunarApplication *application;
+  ThunarFile        *file;
+  ThunarFile        *template_file;
+  GdkScreen         *screen;
+
+  /* parse uri and display parameters */
+  if (!thunar_dbus_service_parse_uri_and_display (dbus_service, parent_directory, display, &file, &screen, error))
+    return FALSE;
+
+  /* try to determine the file for the template URI */
+  template_file = thunar_file_get_for_uri (template_uri, error);
+  if(template_file == NULL)
+    return FALSE;
+
+  /* popup a new window for the folder */
+  application = thunar_application_get ();
+  thunar_application_create_file_from_template (application, file, template_file, screen, startup_id);
+  g_object_unref (G_OBJECT (application));
+
+  /* cleanup */
+  g_object_unref (G_OBJECT (screen));
+  g_object_unref (G_OBJECT (file));
+
+  return TRUE;
+}
+
+
+
 static gboolean
 thunar_dbus_service_transfer_files (ThunarDBusTransferMode transfer_mode,
                                     const gchar           *working_directory,
@@ -1119,3 +1322,12 @@ thunar_dbus_service_terminate (ThunarDBusService *dbus_service,
   /* we cannot fail */
   return TRUE;
 }
+
+
+
+gboolean
+thunar_dbus_service_has_connection (ThunarDBusService *dbus_service)
+{
+  _thunar_return_val_if_fail (THUNAR_IS_DBUS_SERVICE (dbus_service), FALSE);
+  return dbus_service->connection != NULL;
+}
