@@ -58,6 +58,7 @@ static void         thunar_details_view_disconnect_ui_manager   (ThunarStandardV
 static GList       *thunar_details_view_get_selected_items      (ThunarStandardView     *standard_view);
 static void         thunar_details_view_select_all              (ThunarStandardView     *standard_view);
 static void         thunar_details_view_unselect_all            (ThunarStandardView     *standard_view);
+static void         thunar_details_view_selection_invert        (ThunarStandardView     *standard_view);
 static void         thunar_details_view_select_path             (ThunarStandardView     *standard_view,
                                                                  GtkTreePath            *path);
 static void         thunar_details_view_set_cursor              (ThunarStandardView     *standard_view,
@@ -97,6 +98,9 @@ static void         thunar_details_view_columns_changed         (ThunarColumnMod
 static void         thunar_details_view_zoom_level_changed      (ThunarDetailsView      *details_view);
 static void         thunar_details_view_action_setup_columns    (GtkAction              *action,
                                                                  ThunarDetailsView      *details_view);
+static gboolean     thunar_details_view_get_fixed_columns       (ThunarDetailsView      *details_view);
+static void         thunar_details_view_set_fixed_columns       (ThunarDetailsView      *details_view,
+                                                                 gboolean                fixed_columns);
 
 
 
@@ -156,6 +160,7 @@ thunar_details_view_class_init (ThunarDetailsViewClass *klass)
   thunarstandard_view_class->get_selected_items = thunar_details_view_get_selected_items;
   thunarstandard_view_class->select_all = thunar_details_view_select_all;
   thunarstandard_view_class->unselect_all = thunar_details_view_unselect_all;
+  thunarstandard_view_class->selection_invert = thunar_details_view_selection_invert;
   thunarstandard_view_class->select_path = thunar_details_view_select_path;
   thunarstandard_view_class->set_cursor = thunar_details_view_set_cursor;
   thunarstandard_view_class->scroll_to_path = thunar_details_view_scroll_to_path;
@@ -459,6 +464,56 @@ thunar_details_view_unselect_all (ThunarStandardView *standard_view)
 
 
 
+static void
+thunar_details_view_selection_invert_foreach (GtkTreeModel *model,
+                                              GtkTreePath  *path,
+                                              GtkTreeIter  *iter,
+                                              gpointer      data)
+{
+  GList      **list = data;
+
+  *list = g_list_prepend (*list, gtk_tree_path_copy (path));
+}
+
+
+
+static void
+thunar_details_view_selection_invert (ThunarStandardView *standard_view)
+{
+  GtkTreeSelection *selection;
+  GList            *selected_paths = NULL;
+  GList            *lp;
+  GtkTreePath      *path;
+
+  _thunar_return_if_fail (THUNAR_IS_DETAILS_VIEW (standard_view));
+
+  selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (GTK_BIN (standard_view)->child));
+
+  /* block updates */
+  g_signal_handlers_block_by_func (selection, thunar_standard_view_selection_changed, standard_view);
+
+  /* get paths of selected files */
+  gtk_tree_selection_selected_foreach (selection, thunar_details_view_selection_invert_foreach, &selected_paths);
+
+  gtk_tree_selection_select_all (selection);
+
+  for (lp = selected_paths; lp != NULL; lp = lp->next)
+    {
+      path = lp->data;
+      gtk_tree_selection_unselect_path (selection, path);
+      gtk_tree_path_free (path);
+    }
+
+  g_list_free (selected_paths);
+
+  /* unblock updates */
+  g_signal_handlers_unblock_by_func (selection, thunar_standard_view_selection_changed, standard_view);
+
+  thunar_standard_view_selection_changed (THUNAR_STANDARD_VIEW (standard_view));
+}
+
+
+
 static void
 thunar_details_view_select_path (ThunarStandardView *standard_view,
                                  GtkTreePath        *path)
@@ -631,15 +686,10 @@ thunar_details_view_button_press_event (GtkTreeView       *tree_view,
               gtk_tree_selection_select_path (selection, path);
             }
           gtk_tree_path_free (path);
+        }
 
-          /* queue the menu popup */
-          thunar_standard_view_queue_popup (THUNAR_STANDARD_VIEW (details_view), event);
-        }
-      else
-        {
-          /* open the context menu */
-          thunar_standard_view_context_menu (THUNAR_STANDARD_VIEW (details_view), event->button, event->time);
-        }
+      /* open the context menu */
+      thunar_standard_view_context_menu (THUNAR_STANDARD_VIEW (details_view), event->button, event->time);
 
       return TRUE;
     }
@@ -791,22 +841,6 @@ thunar_details_view_action_setup_columns (GtkAction         *action,
 
 
 
-/**
- * thunar_details_view_new:
- *
- * Allocates a new #ThunarDetailsView instance, which is not
- * associated with any #ThunarListModel yet.
- *
- * Return value: the newly allocated #ThunarDetailsView instance.
- **/
-GtkWidget*
-thunar_details_view_new (void)
-{
-  return g_object_new (THUNAR_TYPE_DETAILS_VIEW, NULL);
-}
-
-
-
 /**
  * thunar_details_view_get_fixed_columns:
  * @details_view : a #ThunarDetailsView.
@@ -815,7 +849,7 @@ thunar_details_view_new (void)
  *
  * Return value: %TRUE if @details_view uses fixed column widths.
  **/
-gboolean
+static gboolean
 thunar_details_view_get_fixed_columns (ThunarDetailsView *details_view)
 {
   _thunar_return_val_if_fail (THUNAR_IS_DETAILS_VIEW (details_view), FALSE);
@@ -831,7 +865,7 @@ thunar_details_view_get_fixed_columns (ThunarDetailsView *details_view)
  *
  * Applies @fixed_columns to @details_view.
  **/
-void
+static void
 thunar_details_view_set_fixed_columns (ThunarDetailsView *details_view,
                                        gboolean           fixed_columns)
 {
@@ -880,5 +914,3 @@ thunar_details_view_set_fixed_columns (ThunarDetailsView *details_view,
       g_object_notify (G_OBJECT (details_view), "fixed-columns");
     }
 }
-
-
