@@ -129,23 +129,6 @@ thunar_g_file_is_home (GFile *file)
 
 
 
-gboolean
-thunar_g_file_is_desktop (GFile *file)
-{
-  GFile   *desktop;
-  gboolean is_desktop;
-
-  _thunar_return_val_if_fail (G_IS_FILE (file), FALSE);
-
-  desktop = thunar_g_file_new_for_desktop ();
-  is_desktop = g_file_equal (desktop, file);
-  g_object_unref (desktop);
-
-  return is_desktop;
-}
-
-
-
 GKeyFile *
 thunar_g_file_query_key_file (GFile              *file,
                               GCancellable       *cancellable,
@@ -236,6 +219,42 @@ thunar_g_file_get_location (GFile *file)
 
 
 
+gchar *
+thunar_g_file_get_display_name (GFile *file)
+{
+  gchar *base_name;
+  gchar *display_name;
+
+  _thunar_return_val_if_fail (G_IS_FILE (file), NULL);
+
+  base_name = g_file_get_basename (file);
+  if (G_LIKELY (base_name != NULL))
+    {
+      if (strcmp (base_name, "/") == 0)
+        {
+          display_name = g_strdup (_("File System"));
+          g_free (base_name);
+        }
+      else if (g_utf8_validate (base_name, -1, NULL))
+       {
+         display_name = base_name;
+       }
+     else
+       {
+         display_name = g_uri_escape_string (base_name, G_URI_RESERVED_CHARS_ALLOWED_IN_PATH, TRUE);
+         g_free (base_name);
+       }
+   }
+ else
+   {
+     display_name = g_strdup ("?");
+   }
+
+  return display_name;
+}
+
+
+
 gboolean
 thunar_g_vfs_is_uri_scheme_supported (const gchar *scheme)
 {
@@ -249,6 +268,9 @@ thunar_g_vfs_is_uri_scheme_supported (const gchar *scheme)
   gvfs = g_vfs_get_default ();
   supported_schemes = g_vfs_get_supported_uri_schemes (gvfs);
 
+  if (supported_schemes == NULL)
+    return FALSE;
+
   for (n = 0; !supported && supported_schemes[n] != NULL; ++n) 
     if (g_strcmp0 (supported_schemes[n], scheme) == 0)
       supported = TRUE;
@@ -309,35 +331,28 @@ thunar_g_file_list_new_from_string (const gchar *string)
 
 
 /**
- * thunar_g_file_list_to_string:
+ * thunar_g_file_list_to_stringv:
  * @list : a list of #GFile<!---->s.
  *
- * Free the returned value using g_free() when you
- * are done with it.
+ * Free the returned value using g_strfreev() when you
+ * are done with it. Useful for gtk_selection_data_set_uris.
  *
- * Return value: the string representation of @list conforming to the
- *               text/uri-list mime type defined in RFC 2483.
+ * Return value: and array of uris.
  **/
-gchar *
-thunar_g_file_list_to_string (GList *list)
+gchar **
+thunar_g_file_list_to_stringv (GList *list)
 {
-  GString *string;
-  gchar   *uri;
-  GList   *lp;
+  gchar **uris;
+  guint   n;
+  GList  *lp;
 
   /* allocate initial string */
-  string = g_string_new (NULL);
+  uris = g_new0 (gchar *, g_list_length (list) + 1);
 
-  for (lp = list; lp != NULL; lp = lp->next)
-    {
-      uri = g_file_get_uri (lp->data);
-      string = g_string_append (string, uri);
-      g_free (uri);
+  for (lp = list, n = 0; lp != NULL; lp = lp->next)
+    uris[n++] = g_file_get_uri (G_FILE (lp->data));
 
-      string = g_string_append (string, "\r\n");
-    }
-
-  return g_string_free (string, FALSE);
+  return uris;
 }
 
 
@@ -402,167 +417,6 @@ thunar_g_file_list_free (GList *list)
 
 
 
-#ifdef HAVE_GIO_UNIX
-static gboolean
-thunar_g_mount_is_internal (GMount *mount)
-{
-  const gchar *point_mount_path;
-  gboolean     is_internal = FALSE;
-  GFile       *root;
-  GList       *lp;
-  GList       *mount_points;
-  gchar       *mount_path;
-
-  _thunar_return_val_if_fail (G_IS_MOUNT (mount), FALSE);
-
-  /* determine the mount path */
-  root = g_mount_get_root (mount);
-  mount_path = g_file_get_path (root);
-  g_object_unref (root);
-
-  /* assume non-internal if we cannot determine the path */
-  if (mount_path == NULL)
-    return FALSE;
-
-  if (g_unix_is_mount_path_system_internal (mount_path))
-    {
-      /* mark as internal */
-      is_internal = TRUE;
-    }
-  else
-    {
-      /* get a list of all mount points */
-      mount_points = g_unix_mount_points_get (NULL);
-
-      /* search for the mount point associated with the mount entry */
-      for (lp = mount_points; !is_internal && lp != NULL; lp = lp->next)
-        {
-          point_mount_path = g_unix_mount_point_get_mount_path (lp->data);
-
-          /* check if this is the mount point we are looking for */
-          if (g_strcmp0 (mount_path, point_mount_path) == 0)
-            {
-              /* mark as internal if the user cannot mount this device */
-              if (!g_unix_mount_point_is_user_mountable (lp->data))
-                is_internal = TRUE;
-            }
-              
-          /* free the mount point, we no longer need it */
-          g_unix_mount_point_free (lp->data);
-        }
-
-      /* free the mount point list */
-      g_list_free (mount_points);
-    }
-
-  g_free (mount_path);
-
-  return is_internal;
-}
-#endif
-
-
-
-gboolean
-thunar_g_volume_is_removable (GVolume *volume)
-{
-  gboolean         can_eject = FALSE;
-  gboolean         can_mount = FALSE;
-  gboolean         can_unmount = FALSE;
-  gboolean         is_removable = FALSE;
-  gboolean         is_internal = FALSE;
-  GDrive          *drive;
-  GMount          *mount;
-
-  _thunar_return_val_if_fail (G_IS_VOLUME (volume), FALSE);
-  
-  /* check if the volume can be ejected */
-  can_eject = g_volume_can_eject (volume);
-
-  /* determine the drive for the volume */
-  drive = g_volume_get_drive (volume);
-  if (drive != NULL)
-    {
-      /*check if the drive media can be removed */
-      is_removable = g_drive_is_media_removable (drive);
-
-      /* release the drive */
-      g_object_unref (drive);
-    }
-
-  /* determine the mount for the volume (if it is mounted at all) */
-  mount = g_volume_get_mount (volume);
-  if (mount != NULL)
-    {
-#ifdef HAVE_GIO_UNIX
-      is_internal = thunar_g_mount_is_internal (mount);
-#endif
-
-      /* check if the volume can be unmounted */
-      can_unmount = g_mount_can_unmount (mount);
-
-      /* release the mount */
-      g_object_unref (mount);
-    }
-
-  /* determine whether the device can be mounted */
-  can_mount = g_volume_can_mount (volume);
-
-  return (!is_internal) && (can_eject || can_unmount || is_removable || can_mount);
-}
-
-
-
-gboolean
-thunar_g_volume_is_mounted (GVolume *volume)
-{
-  gboolean is_mounted = FALSE;
-  GMount  *mount;
-
-  _thunar_return_val_if_fail (G_IS_VOLUME (volume), FALSE);
-
-  /* determine the mount for this volume (if it is mounted at all) */
-  mount = g_volume_get_mount (volume);
-  if (mount != NULL)
-    {
-      is_mounted = TRUE;
-      g_object_unref (mount);
-    }
-
-  return is_mounted;
-}
-
-
-
-gboolean 
-thunar_g_volume_is_present (GVolume *volume)
-{
-  gboolean has_media = FALSE;
-  gboolean is_shadowed = FALSE;
-  GDrive  *drive;
-  GMount  *mount;
-
-  _thunar_return_val_if_fail (G_IS_VOLUME (volume), FALSE);
-
-  drive = g_volume_get_drive (volume);
-  if (drive != NULL)
-    {
-      has_media = g_drive_has_media (drive);
-      g_object_unref (drive);
-    }
-
-  mount = g_volume_get_mount (volume);
-  if (mount != NULL)
-    {
-      is_shadowed = g_mount_is_shadowed (mount);
-      g_object_unref (mount);
-    }
-
-  return has_media && !is_shadowed;
-}
-
-
-
 gboolean
 thunar_g_app_info_launch (GAppInfo          *info,
                           GFile             *working_directory,
