@@ -33,7 +33,7 @@ GFile    *thunar_g_file_new_for_user_special_dir (GUserDirectory        dir);
                                                                       
 gboolean  thunar_g_file_is_root                  (GFile                *file);
 gboolean  thunar_g_file_is_trashed               (GFile                *file);
-gboolean  thunar_g_file_is_desktop               (GFile                *file);
+gboolean  thunar_g_file_is_home                  (GFile                *file);
                                                                       
 GKeyFile *thunar_g_file_query_key_file           (GFile                *file,
                                                   GCancellable         *cancellable,
@@ -44,6 +44,8 @@ gboolean  thunar_g_file_write_key_file           (GFile                *file,
                                                   GError              **error);
                                                                       
 gchar    *thunar_g_file_get_location             (GFile                *file);
+
+gchar    *thunar_g_file_get_display_name         (GFile                *file);
                                                                       
 gboolean  thunar_g_vfs_is_uri_scheme_supported   (const gchar          *scheme);
 
@@ -58,6 +60,7 @@ gboolean  thunar_g_vfs_is_uri_scheme_supported   (const gchar          *scheme);
 GType     thunar_g_file_list_get_type               (void);
 
 GList    *thunar_g_file_list_new_from_string        (const gchar       *string);
+gchar   **thunar_g_file_list_to_stringv             (GList             *list);
 gchar    *thunar_g_file_list_to_string              (GList             *list);
 GList    *thunar_g_file_list_append                 (GList             *list,
                                                      GFile             *file);
@@ -66,10 +69,6 @@ GList    *thunar_g_file_list_prepend                (GList             *list,
 GList    *thunar_g_file_list_copy                   (GList             *list);
 void      thunar_g_file_list_free                   (GList             *list);
 
-gboolean  thunar_g_volume_is_removable              (GVolume           *volume);
-gboolean  thunar_g_volume_is_mounted                (GVolume           *volume);
-gboolean  thunar_g_volume_is_present                (GVolume           *volume);
-
 gboolean  thunar_g_app_info_launch                  (GAppInfo          *info,
                                                      GFile             *working_directory,
                                                      GList             *path_list,
