@@ -459,7 +459,7 @@ thunar_icon_factory_load_from_file (ThunarIconFactory *factory,
       if (G_LIKELY (width > max_width || height > max_height))
         {
           /* scale down to the required size */
-          tmp = exo_gdk_pixbuf_scale_down (pixbuf, TRUE, max_height, max_height);
+          tmp = exo_gdk_pixbuf_scale_down (pixbuf, TRUE, MAX (1, max_height), MAX (1, max_height));
           g_object_unref (G_OBJECT (pixbuf));
           pixbuf = tmp;
         }
@@ -698,7 +698,8 @@ thunar_icon_factory_get_for_icon_theme (GtkIconTheme *icon_theme)
 
       /* connect the "show-thumbnails" property to the global preference */
       factory->preferences = thunar_preferences_get ();
-      exo_mutual_binding_new (G_OBJECT (factory->preferences), "misc-show-thumbnails", G_OBJECT (factory), "show-thumbnails");
+      exo_binding_new (G_OBJECT (factory->preferences), "misc-show-thumbnails",
+                       G_OBJECT (factory), "show-thumbnails");
     }
   else
     {
@@ -710,23 +711,6 @@ thunar_icon_factory_get_for_icon_theme (GtkIconTheme *icon_theme)
 
 
 
-/**
- * thunar_icon_factory_get_icon_theme:
- * @factory : a #ThunarIconFactory instance.
- *
- * Returns the #GtkIconTheme associated with @factory.
- *
- * Return value: the #GtkIconTheme associated with @factory.
- **/
-GtkIconTheme*
-thunar_icon_factory_get_icon_theme (const ThunarIconFactory *factory)
-{
-  _thunar_return_val_if_fail (THUNAR_IS_ICON_FACTORY (factory), NULL);
-  return factory->icon_theme;
-}
-
-
-
 /**
  * thunar_icon_factory_load_icon:
  * @factory       : a #ThunarIconFactory instance.
