@@ -1,4 +1,4 @@
-/* $Id: thunar-icon-renderer.c 23612 2006-11-03 16:28:03Z benny $ */
+/* $Id$ */
 /*-
  * Copyright (c) 2005-2006 Benedikt Meurer <benny@xfce.org>
  *
@@ -41,8 +41,6 @@ enum
 
 
 
-static void thunar_icon_renderer_class_init    (ThunarIconRendererClass *klass);
-static void thunar_icon_renderer_init          (ThunarIconRenderer      *icon_renderer);
 static void thunar_icon_renderer_finalize      (GObject                 *object);
 static void thunar_icon_renderer_get_property  (GObject                 *object,
                                                 guint                    prop_id,
@@ -69,36 +67,7 @@ static void thunar_icon_renderer_render        (GtkCellRenderer         *rendere
 
 
 
-static GObjectClass *thunar_icon_renderer_parent_class;
-
-
-
-GType
-thunar_icon_renderer_get_type (void)
-{
-  static GType type = G_TYPE_INVALID;
-
-  if (G_UNLIKELY (type == G_TYPE_INVALID))
-    {
-      static const GTypeInfo info =
-      {
-        sizeof (ThunarIconRendererClass),
-        NULL,
-        NULL,
-        (GClassInitFunc) thunar_icon_renderer_class_init,
-        NULL,
-        NULL,
-        sizeof (ThunarIconRenderer),
-        0,
-        (GInstanceInitFunc) thunar_icon_renderer_init,
-        NULL,
-      };
-
-      type = g_type_register_static (GTK_TYPE_CELL_RENDERER, I_("ThunarIconRenderer"), &info, 0);
-    }
-
-  return type;
-}
+G_DEFINE_TYPE (ThunarIconRenderer, thunar_icon_renderer, GTK_TYPE_CELL_RENDERER)
 
 
 
@@ -108,9 +77,6 @@ thunar_icon_renderer_class_init (ThunarIconRendererClass *klass)
   GtkCellRendererClass *gtkcell_renderer_class;
   GObjectClass         *gobject_class;
 
-  /* determine the parent type class */
-  thunar_icon_renderer_parent_class = g_type_class_peek_parent (klass);
-
   gobject_class = G_OBJECT_CLASS (klass);
   gobject_class->finalize = thunar_icon_renderer_finalize;
   gobject_class->get_property = thunar_icon_renderer_get_property;
@@ -367,6 +333,8 @@ thunar_icon_renderer_render (GtkCellRenderer     *renderer,
   GList                  *lp;
   gint                    max_emblems;
   gint                    position;
+  cairo_t                *cr;
+  gdouble                 alpha;
 
   if (G_UNLIKELY (icon_renderer->file == NULL))
     return;
@@ -400,7 +368,7 @@ thunar_icon_renderer_render (GtkCellRenderer     *renderer,
   if (G_UNLIKELY (icon_area.width > cell_area->width || icon_area.height > cell_area->height))
     {
       /* scale down to fit */
-      temp = exo_gdk_pixbuf_scale_down (icon, TRUE, cell_area->width, cell_area->height);
+      temp = exo_gdk_pixbuf_scale_down (icon, TRUE, MAX (1, cell_area->width), MAX (1, cell_area->height));
       g_object_unref (G_OBJECT (icon));
       icon = temp;
 
@@ -412,6 +380,9 @@ thunar_icon_renderer_render (GtkCellRenderer     *renderer,
   icon_area.x = cell_area->x + (cell_area->width - icon_area.width) / 2;
   icon_area.y = cell_area->y + (cell_area->height - icon_area.height) / 2;
 
+  /* create the context */
+  cr = gdk_cairo_create (window);
+
   /* check whether the icon is affected by the expose event */
   if (gdk_rectangle_intersect (expose_area, &icon_area, &draw_area))
     {
@@ -420,16 +391,16 @@ thunar_icon_renderer_render (GtkCellRenderer     *renderer,
       if (thunar_clipboard_manager_has_cutted_file (clipboard, icon_renderer->file))
         {
           /* 50% translucent for cutted files */
-          temp = exo_gdk_pixbuf_lucent (icon, 50);
-          g_object_unref (G_OBJECT (icon));
-          icon = temp;
+          alpha = 0.50;
         }
       else if (thunar_file_is_hidden (icon_renderer->file))
         {
           /* 75% translucent for hidden files */
-          temp = exo_gdk_pixbuf_lucent (icon, 75);
-          g_object_unref (G_OBJECT (icon));
-          icon = temp;
+          alpha = 0.75;
+        }
+      else
+        {
+          alpha = 1.00;
         }
       g_object_unref (G_OBJECT (clipboard));
 
@@ -438,7 +409,7 @@ thunar_icon_renderer_render (GtkCellRenderer     *renderer,
         {
           if ((flags & GTK_CELL_RENDERER_SELECTED) != 0)
             {
-              state = GTK_WIDGET_HAS_FOCUS (widget) ? GTK_STATE_SELECTED : GTK_STATE_ACTIVE;
+              state = gtk_widget_has_focus (widget) ? GTK_STATE_SELECTED : GTK_STATE_ACTIVE;
               temp = exo_gdk_pixbuf_colorize (icon, &widget->style->base[state]);
               g_object_unref (G_OBJECT (icon));
               icon = temp;
@@ -453,7 +424,7 @@ thunar_icon_renderer_render (GtkCellRenderer     *renderer,
         }
 
       /* check if we should render an insensitive icon */
-      if (G_UNLIKELY (GTK_WIDGET_STATE (widget) == GTK_STATE_INSENSITIVE || !renderer->sensitive))
+      if (G_UNLIKELY (gtk_widget_get_state (widget) == GTK_STATE_INSENSITIVE || !renderer->sensitive))
         {
           /* allocate an icon source */
           icon_source = gtk_icon_source_new ();
@@ -472,10 +443,9 @@ thunar_icon_renderer_render (GtkCellRenderer     *renderer,
         }
 
       /* render the invalid parts of the icon */
-      gdk_draw_pixbuf (window, widget->style->black_gc, icon,
-                       draw_area.x - icon_area.x, draw_area.y - icon_area.y,
-                       draw_area.x, draw_area.y, draw_area.width, draw_area.height,
-                       GDK_RGB_DITHER_NORMAL, 0, 0);
+      gdk_cairo_set_source_pixbuf (cr, icon, icon_area.x, icon_area.y);
+      gdk_cairo_rectangle (cr, &draw_area);
+      cairo_paint_with_alpha (cr, alpha);
     }
 
   /* release the file's icon */
@@ -504,10 +474,10 @@ thunar_icon_renderer_render (GtkCellRenderer     *renderer,
               emblem_area.height = gdk_pixbuf_get_height (emblem);
 
               /* shrink insane emblems */
-              if (G_UNLIKELY (MAX (emblem_area.width, emblem_area.height) > MIN ((2 * icon_renderer->size) / 3, 36)))
+              if (G_UNLIKELY (MAX (emblem_area.width, emblem_area.height) > (gint) MIN ((2 * icon_renderer->size) / 3, 32)))
                 {
                   /* scale down the emblem */
-                  temp = exo_gdk_pixbuf_scale_ratio (emblem, MIN ((2 * icon_renderer->size) / 3, 36));
+                  temp = exo_gdk_pixbuf_scale_ratio (emblem, MIN ((2 * icon_renderer->size) / 3, 32));
                   g_object_unref (G_OBJECT (emblem));
                   emblem = temp;
 
@@ -554,10 +524,10 @@ thunar_icon_renderer_render (GtkCellRenderer     *renderer,
               /* render the emblem */
               if (gdk_rectangle_intersect (expose_area, &emblem_area, &draw_area))
                 {
-                  gdk_draw_pixbuf (window, widget->style->black_gc, emblem,
-                                   draw_area.x - emblem_area.x, draw_area.y - emblem_area.y,
-                                   draw_area.x, draw_area.y, draw_area.width, draw_area.height,
-                                   GDK_RGB_DITHER_NORMAL, 0, 0);
+                  /* render the invalid parts of the icon */
+                  gdk_cairo_set_source_pixbuf (cr, emblem, emblem_area.x, emblem_area.y);
+                  gdk_cairo_rectangle (cr, &draw_area);
+                  cairo_paint (cr);
                 }
 
               /* release the emblem */
@@ -572,6 +542,9 @@ thunar_icon_renderer_render (GtkCellRenderer     *renderer,
         }
     }
 
+  /* destroy the context */
+  cairo_destroy (cr);
+
   /* release our reference on the icon factory */
   g_object_unref (G_OBJECT (icon_factory));
 }
