@@ -96,18 +96,33 @@ _thunar_io_jobs_create (ThunarJob   *job,
   GList             *lp;
   gchar             *base_name;
   gchar             *display_name;
+  GFile             *template_file;
+  GFileInputStream  *template_stream = NULL;
   
   _thunar_return_val_if_fail (THUNAR_IS_JOB (job), FALSE);
   _thunar_return_val_if_fail (param_values != NULL, FALSE);
-  _thunar_return_val_if_fail (param_values->n_values == 1, FALSE);
+  _thunar_return_val_if_fail (param_values->n_values == 2, FALSE);
   _thunar_return_val_if_fail (error == NULL || *error == NULL, FALSE);
 
   /* get the file list */
   file_list = g_value_get_boxed (g_value_array_get_nth (param_values, 0));
+  template_file = g_value_get_object (g_value_array_get_nth (param_values, 1));
 
   /* we know the total amount of files to be processed */
   thunar_job_set_total_files (THUNAR_JOB (job), file_list);
 
+  /* check if we need to open the template */
+  if (template_file != NULL)
+    {
+      /* open read stream to feed in the new files */
+      template_stream = g_file_read (template_file, exo_job_get_cancellable (EXO_JOB (job)), &err);
+      if (G_UNLIKELY (template_stream == NULL))
+        {
+          g_propagate_error (error, err);
+          return FALSE;
+        }
+    }
+
   /* iterate over all files in the list */
   for (lp = file_list; 
        err == NULL && lp != NULL && !exo_job_is_cancelled (EXO_JOB (job)); 
@@ -197,9 +212,24 @@ again:
             }
         }
       else
-        g_object_unref (stream);
+        {
+          if (template_stream != NULL)
+            {
+              /* write the template into the new file */
+              g_output_stream_splice (G_OUTPUT_STREAM (stream),
+                                      G_INPUT_STREAM (template_stream),
+                                      G_OUTPUT_STREAM_SPLICE_CLOSE_TARGET,
+                                      exo_job_get_cancellable (EXO_JOB (job)),
+                                      NULL);
+            }
+
+          g_object_unref (stream);
+        }
     }
 
+  if (template_stream != NULL)
+    g_object_unref (template_stream);
+
   /* check if we have failed */
   if (err != NULL)
     {
@@ -220,10 +250,12 @@ again:
 
 
 ThunarJob *
-thunar_io_jobs_create_files (GList *file_list)
+thunar_io_jobs_create_files (GList *file_list,
+                             GFile *template_file)
 {
-  return thunar_simple_job_launch (_thunar_io_jobs_create, 1,
-                                   THUNAR_TYPE_G_FILE_LIST, file_list);
+  return thunar_simple_job_launch (_thunar_io_jobs_create, 2,
+                                   THUNAR_TYPE_G_FILE_LIST, file_list,
+                                   G_TYPE_FILE, template_file);
 }
 
 
@@ -958,25 +990,20 @@ retry_chown:
 
 
 ThunarJob *
-thunar_io_jobs_change_group (GFile    *file,
+thunar_io_jobs_change_group (GList    *files,
                              guint32   gid,
                              gboolean  recursive)
 {
-  GList file_list;
+  _thunar_return_val_if_fail (files != NULL, NULL);
 
-  _thunar_return_val_if_fail (G_IS_FILE (file), NULL);
+  /* files are released when the list if destroyed */
+  g_list_foreach (files, (GFunc) g_object_ref, NULL);
 
-  file_list.data = g_object_ref (file);
-  file_list.next = NULL; 
-  file_list.prev = NULL;
-  
   return thunar_simple_job_launch (_thunar_io_jobs_chown, 4,
-                                   THUNAR_TYPE_G_FILE_LIST, &file_list,
+                                   THUNAR_TYPE_G_FILE_LIST, files,
                                    G_TYPE_INT, -1,
                                    G_TYPE_INT, (gint) gid,
                                    G_TYPE_BOOLEAN, recursive);
-
-  g_object_unref (file_list.data);
 }
 
 
@@ -1066,12 +1093,15 @@ retry_chown:
        * information) into account */
       new_mode = ((old_mode & ~mask) | mode) & 07777;
 
-      /* try to change the file mode */
-      g_file_set_attribute_uint32 (lp->data,
-                                   G_FILE_ATTRIBUTE_UNIX_MODE, new_mode,
-                                   G_FILE_QUERY_INFO_NOFOLLOW_SYMLINKS,
-                                   exo_job_get_cancellable (EXO_JOB (job)),
-                                   &err);
+      if (old_mode != new_mode)
+        {
+          /* try to change the file mode */
+          g_file_set_attribute_uint32 (lp->data,
+                                       G_FILE_ATTRIBUTE_UNIX_MODE, new_mode,
+                                       G_FILE_QUERY_INFO_NOFOLLOW_SYMLINKS,
+                                       exo_job_get_cancellable (EXO_JOB (job)),
+                                       &err);
+        }
 
       /* check if there was a recoverable error */
       if (err != NULL && !exo_job_is_cancelled (EXO_JOB (job)))
@@ -1112,30 +1142,25 @@ retry_chown:
 
 
 ThunarJob *
-thunar_io_jobs_change_mode (GFile         *file,
+thunar_io_jobs_change_mode (GList         *files,
                             ThunarFileMode dir_mask,
                             ThunarFileMode dir_mode,
                             ThunarFileMode file_mask,
                             ThunarFileMode file_mode,
                             gboolean       recursive)
 {
-  GList file_list;
+  _thunar_return_val_if_fail (files != NULL, NULL);
 
-  _thunar_return_val_if_fail (G_IS_FILE (file), NULL);
+  /* files are released when the list if destroyed */
+  g_list_foreach (files, (GFunc) g_object_ref, NULL);
 
-  file_list.data = g_object_ref (file);
-  file_list.next = NULL; 
-  file_list.prev = NULL;
-  
   return thunar_simple_job_launch (_thunar_io_jobs_chmod, 6,
-                                   THUNAR_TYPE_G_FILE_LIST, &file_list,
+                                   THUNAR_TYPE_G_FILE_LIST, files,
                                    THUNAR_TYPE_FILE_MODE, dir_mask,
                                    THUNAR_TYPE_FILE_MODE, dir_mode,
                                    THUNAR_TYPE_FILE_MODE, file_mask,
                                    THUNAR_TYPE_FILE_MODE, file_mode,
                                    G_TYPE_BOOLEAN, recursive);
-
-  g_object_unref (file_list.data);
 }
 
 
