@@ -41,6 +41,7 @@
 #include <thunar/thunar-private.h>
 #include <thunar/thunar-sendto-model.h>
 #include <thunar/thunar-stock.h>
+#include <thunar/thunar-device-monitor.h>
 
 
 
@@ -100,7 +101,7 @@ static void                    thunar_launcher_action_open_in_new_window  (GtkAc
                                                                            ThunarLauncher           *launcher);
 static void                    thunar_launcher_action_sendto_desktop      (GtkAction                *action,
                                                                            ThunarLauncher           *launcher);
-static void                    thunar_launcher_action_sendto_volume       (GtkAction                *action,
+static void                    thunar_launcher_action_sendto_device       (GtkAction                *action,
                                                                            ThunarLauncher           *launcher);
 static void                    thunar_launcher_widget_destroyed           (ThunarLauncher           *launcher,
                                                                            GtkWidget                *widget);
@@ -116,7 +117,7 @@ static void                    thunar_launcher_poke_files_finish          (Thuna
                                                                            gpointer                  user_data);
 static ThunarLauncherPokeData *thunar_launcher_poke_data_new              (GList                    *files);
 static void                    thunar_launcher_poke_data_free             (ThunarLauncherPokeData   *data);
-
+static GtkWidget              *thunar_launcher_get_widget                 (const ThunarLauncher     *launcher);
 
 
 struct _ThunarLauncherClass
@@ -144,7 +145,7 @@ struct _ThunarLauncher
 
   GtkWidget              *widget;
 
-  GVolumeMonitor         *volume_monitor;
+  ThunarDeviceMonitor    *device_monitor;
   ThunarSendtoModel      *sendto_model;
   gint                    sendto_idle_id;
 };
@@ -262,10 +263,10 @@ thunar_launcher_init (ThunarLauncher *launcher)
   /* setup the "Send To" support */
   launcher->sendto_model = thunar_sendto_model_get_default ();
 
-  /* the "Send To" menu also displays removable devices from the volume monitor */
-  launcher->volume_monitor = g_volume_monitor_get ();
-  g_signal_connect_swapped (launcher->volume_monitor, "volume-added", G_CALLBACK (thunar_launcher_update), launcher);
-  g_signal_connect_swapped (launcher->volume_monitor, "volume-removed", G_CALLBACK (thunar_launcher_update), launcher);
+  /* the "Send To" menu also displays removable devices from the device monitor */
+  launcher->device_monitor = thunar_device_monitor_get ();
+  g_signal_connect_swapped (launcher->device_monitor, "device-added", G_CALLBACK (thunar_launcher_update), launcher);
+  g_signal_connect_swapped (launcher->device_monitor, "device-removed", G_CALLBACK (thunar_launcher_update), launcher);
 }
 
 
@@ -305,9 +306,9 @@ thunar_launcher_finalize (GObject *object)
   /* release the reference on the action group */
   g_object_unref (launcher->action_group);
 
-  /* disconnect from the volume monitor used for the "Send To" menu */
-  g_signal_handlers_disconnect_by_func (launcher->volume_monitor, thunar_launcher_update, launcher);
-  g_object_unref (launcher->volume_monitor);
+  /* disconnect from the device monitor used for the "Send To" menu */
+  g_signal_handlers_disconnect_by_func (launcher->device_monitor, thunar_launcher_update, launcher);
+  g_object_unref (launcher->device_monitor);
 
   /* release the reference on the sendto model */
   g_object_unref (launcher->sendto_model);
@@ -521,17 +522,16 @@ static void
 thunar_launcher_execute_files (ThunarLauncher *launcher,
                                GList          *files)
 {
-  GdkScreen *screen;
-  GError    *error = NULL;
-  GList     *lp;
-
-  /* determine the screen on which to run the file(s) */
-  screen = (launcher->widget != NULL) ? gtk_widget_get_screen (launcher->widget) : NULL;
+  GError *error = NULL;
+  GFile  *working_directory;
+  GList  *lp;
 
   /* execute all selected files */
   for (lp = files; lp != NULL; lp = lp->next)
     {
-      if (!thunar_file_execute (lp->data, screen, NULL, &error))
+      working_directory = thunar_file_get_file (launcher->current_directory);
+
+      if (!thunar_file_execute (lp->data, working_directory, launcher->widget, NULL, &error))
         {
           /* display an error message to the user */
           thunar_dialogs_show_error (launcher->widget, error, _("Failed to execute file \"%s\""), thunar_file_get_display_name (lp->data));
@@ -543,17 +543,28 @@ thunar_launcher_execute_files (ThunarLauncher *launcher,
 
 
 
+static guint
+thunar_launcher_g_app_info_hash (gconstpointer app_info)
+{
+  return 0;
+}
+
+
+
 static void
 thunar_launcher_open_files (ThunarLauncher *launcher,
                             GList          *files)
 {
-  GAppInfo   *app_info;
   GHashTable *applications;
+  GAppInfo   *app_info;
   GList      *file_list;
   GList      *lp;
 
-  /* allocate a hash table to associate applications to URIs */
-  applications = g_hash_table_new_full (g_direct_hash,
+  /* allocate a hash table to associate applications to URIs. since GIO allocates
+   * new GAppInfo objects every time, g_direct_hash does not work. we therefor use
+   * a fake hash function to always hit the collision list of the hash table and
+   * avoid storing multiple equal GAppInfos by means of g_app_info_equal(). */
+  applications = g_hash_table_new_full (thunar_launcher_g_app_info_hash,
                                         (GEqualFunc) g_app_info_equal,
                                         (GDestroyNotify) g_object_unref,
                                         (GDestroyNotify) thunar_g_file_list_free);
@@ -616,6 +627,8 @@ thunar_launcher_open_paths (GAppInfo       *app_info,
   /* create launch context */
   context = gdk_app_launch_context_new ();
   gdk_app_launch_context_set_screen (context, screen);
+  gdk_app_launch_context_set_timestamp (context, gtk_get_current_event_time ());
+  gdk_app_launch_context_set_icon (context, g_app_info_get_icon (app_info));
 
   /* determine the working directory */
   if (launcher->current_directory != NULL)
@@ -715,8 +728,6 @@ thunar_launcher_update (ThunarLauncher *launcher)
 {
   const gchar  *context_menu_path;
   const gchar  *file_menu_path;
-  GtkWidget    *menu_item;
-  GtkWidget    *image;
   GtkAction    *action;
   gboolean      default_is_open_with_other = FALSE;
   GList        *applications;
@@ -725,7 +736,6 @@ thunar_launcher_update (ThunarLauncher *launcher)
   gchar        *tooltip;
   gchar        *label;
   gchar        *name;
-  gchar        *ui_path;
   gint          n_directories = 0;
   gint          n_executables = 0;
   gint          n_regulars = 0;
@@ -949,6 +959,7 @@ thunar_launcher_update (ThunarLauncher *launcher)
 
               /* allocate a new action for the application */
               action = gtk_action_new (name, label, tooltip, NULL);
+              gtk_action_set_gicon (action, g_app_info_get_icon (lp->data));
               gtk_action_group_add_action (launcher->action_group, action);
               g_object_set_qdata_full (G_OBJECT (action), thunar_launcher_handler_quark, lp->data, g_object_unref);
               g_signal_connect (G_OBJECT (action), "activate", G_CALLBACK (thunar_launcher_action_open), launcher);
@@ -960,23 +971,6 @@ thunar_launcher_update (ThunarLauncher *launcher)
                                      GTK_UI_MANAGER_MENUITEM, FALSE);
               g_object_unref (G_OBJECT (action));
 
-              /* FIXME There's no API for creating GtkActions using GIcon in GTK+ 2.14. A "gicon" property
-               * has been added to GtkAction in GTK+ 2.16 though. For now, this hack will have to do: */
-
-              ui_path = g_strconcat (file_menu_path, "/", name, NULL);
-              menu_item = gtk_ui_manager_get_widget (launcher->ui_manager, ui_path);
-              image = gtk_image_menu_item_get_image (GTK_IMAGE_MENU_ITEM (menu_item));
-              gtk_image_set_from_gicon (GTK_IMAGE (image), g_app_info_get_icon (lp->data), GTK_ICON_SIZE_MENU);
-              g_free (ui_path);
-              	
-              ui_path = g_strconcat (context_menu_path, "/", name, NULL);
-              menu_item = gtk_ui_manager_get_widget (launcher->ui_manager, ui_path);
-              image = gtk_image_menu_item_get_image (GTK_IMAGE_MENU_ITEM (menu_item));
-              gtk_image_set_from_gicon (GTK_IMAGE (image), g_app_info_get_icon (lp->data), GTK_ICON_SIZE_MENU);
-              g_free (ui_path);
-
-              /* FIXME End of the hack */
-
               /* cleanup */
               g_free (tooltip);
               g_free (label);
@@ -1326,66 +1320,56 @@ thunar_launcher_poke_data_free (ThunarLauncherPokeData *data)
 
 
 static void
-thunar_launcher_sendto_volume (ThunarLauncher *launcher,
-                               GVolume        *volume,
+thunar_launcher_sendto_device (ThunarLauncher *launcher,
+                               ThunarDevice   *device,
                                GList          *files)
 {
   ThunarApplication *application;
-  GMount            *mount;
   GFile             *mount_point;
 
   _thunar_return_if_fail (THUNAR_IS_LAUNCHER (launcher));
-  _thunar_return_if_fail (G_IS_VOLUME (volume));
+  _thunar_return_if_fail (THUNAR_IS_DEVICE (device));
 
-  if (!thunar_g_volume_is_mounted (volume))
+  if (!thunar_device_is_mounted (device))
     return;
   
-  mount = g_volume_get_mount (volume);
-  if (mount != NULL)
+  mount_point = thunar_device_get_root (device);
+  if (mount_point != NULL)
     {
-      mount_point = g_mount_get_root (mount);
-      
-      /* copy the files onto the specified volume */
+      /* copy the files onto the specified device */
       application = thunar_application_get ();
       thunar_application_copy_into (application, launcher->widget, files, mount_point, NULL);
       g_object_unref (application);
 
       g_object_unref (mount_point);
-      g_object_unref (mount);
     }
 }
 
 
 
 static void
-thunar_launcher_sendto_mount_finish (GObject      *object,
-                                     GAsyncResult *result,
+thunar_launcher_sendto_mount_finish (ThunarDevice *device,
+                                     const GError *error,
                                      gpointer      user_data)
 {
   ThunarLauncherMountData *data = user_data;
-  GVolume                 *volume = G_VOLUME (object);
-  GError                  *error = NULL;
-  gchar                   *volume_name;
+  gchar                   *device_name;
 
-  _thunar_return_if_fail (G_IS_VOLUME (object));
-  _thunar_return_if_fail (G_IS_ASYNC_RESULT (result));
+  _thunar_return_if_fail (THUNAR_IS_DEVICE (device));
   _thunar_return_if_fail (user_data != NULL);
   _thunar_return_if_fail (THUNAR_IS_LAUNCHER (data->launcher));
 
-  if (!g_volume_mount_finish (volume, result, &error))
+  if (error != NULL)
     {
-      /* tell the user that we were unable to mount the volume, which is 
+      /* tell the user that we were unable to mount the device, which is 
        * required to send files to it */
-      volume_name = g_volume_get_name (volume);
-      thunar_dialogs_show_error (data->launcher->widget, error, _("Failed to mount \"%s\""), 
-                                 volume_name);
-      g_free (volume_name);
-
-      g_error_free (error);
+      device_name = thunar_device_get_name (device);
+      thunar_dialogs_show_error (data->launcher->widget, error, _("Failed to mount \"%s\""), device_name);
+      g_free (device_name);
     }
   else
     {
-      thunar_launcher_sendto_volume (data->launcher, volume, data->files);
+      thunar_launcher_sendto_device (data->launcher, device, data->files);
     }
 
   thunar_launcher_mount_data_free (data);
@@ -1394,13 +1378,13 @@ thunar_launcher_sendto_mount_finish (GObject      *object,
 
 
 static void
-thunar_launcher_action_sendto_volume (GtkAction      *action,
+thunar_launcher_action_sendto_device (GtkAction      *action,
                                       ThunarLauncher *launcher)
 {
   ThunarLauncherMountData *data;
   GMountOperation         *mount_operation;
   GtkWidget               *window;
-  GVolume                 *volume;
+  ThunarDevice            *device;
   GList                   *files;
 
   _thunar_return_if_fail (GTK_IS_ACTION (action));
@@ -1411,13 +1395,13 @@ thunar_launcher_action_sendto_volume (GtkAction      *action,
   if (G_UNLIKELY (files == NULL))
     return;
 
-  /* determine the volume to which to send */
-  volume = g_object_get_qdata (G_OBJECT (action), thunar_launcher_handler_quark);
-  if (G_UNLIKELY (volume == NULL))
+  /* determine the device to which to send */
+  device = g_object_get_qdata (G_OBJECT (action), thunar_launcher_handler_quark);
+  if (G_UNLIKELY (device == NULL))
     return;
 
-  /* make sure to mount the volume first, if it's not already mounted */
-  if (!thunar_g_volume_is_mounted (volume))
+  /* make sure to mount the device first, if it's not already mounted */
+  if (!thunar_device_is_mounted (device))
     {
       /* determine the toplevel window */
       window = gtk_widget_get_toplevel (launcher->widget);
@@ -1428,13 +1412,18 @@ thunar_launcher_action_sendto_volume (GtkAction      *action,
       /* allocate a GTK+ mount operation */
       mount_operation = gtk_mount_operation_new (GTK_WINDOW (window));
 
-      /* try to mount the volume and later start sending the files */
-      g_volume_mount (volume, G_MOUNT_MOUNT_NONE, mount_operation, NULL,
-                      thunar_launcher_sendto_mount_finish, data);
+      /* try to mount the device and later start sending the files */
+      thunar_device_mount (device,
+                           mount_operation,
+                           NULL,
+                           thunar_launcher_sendto_mount_finish,
+                           data);
+
+      g_object_unref (mount_operation);
     }
   else
     {
-      thunar_launcher_sendto_volume (launcher, volume, files);
+      thunar_launcher_sendto_device (launcher, device, files);
     }
 
   /* cleanup */
@@ -1463,19 +1452,19 @@ thunar_launcher_sendto_idle (gpointer user_data)
   ThunarLauncher *launcher = THUNAR_LAUNCHER (user_data);
   const gchar    *label;
   GtkAction      *action;
-  GtkWidget      *image;
-  GtkWidget      *menu_item;
   gboolean        linkable = TRUE;
   GIcon          *icon;
   GList          *handlers;
-  GList          *volumes;
+  GList          *devices;
   GList          *lp;
   gchar          *name;
   gchar          *tooltip;
-  gchar          *ui_path;
-  gchar          *volume_name;
+  gchar          *device_name;
   gint            n_selected_files;
   gint            n = 0;
+  gboolean        got_devices = FALSE;
+  const gchar    *file_menu_path;
+  const gchar    *context_menu_path;
 
   /* verify that we have an UI manager */
   if (launcher->ui_manager == NULL)
@@ -1517,74 +1506,66 @@ thunar_launcher_sendto_idle (gpointer user_data)
       if (G_UNLIKELY (launcher->ui_addons_merge_id == 0))
         launcher->ui_addons_merge_id = gtk_ui_manager_new_merge_id (launcher->ui_manager);
 
-      /* determine the currently active volumes */
-      volumes = g_volume_monitor_get_volumes (launcher->volume_monitor);
+      /* determine the currently active devices */
+      devices = thunar_device_monitor_get_devices (launcher->device_monitor);
+      got_devices = (devices != NULL);
+
+      /* paths in ui */
+      file_menu_path = "/main-menu/file-menu/sendto-menu/placeholder-sendto-actions";
+      context_menu_path = "/file-context-menu/sendto-menu/placeholder-sendto-actions";
 
       /* add removable (and writable) drives and media */
-      for (lp = volumes; lp != NULL; lp = lp->next, ++n)
+      for (lp = devices; lp != NULL; lp = lp->next, ++n)
         {
-          /* skip non-removable or disc media (CD-ROMs aren't writable by Thunar) */
-          /* TODO skip non-writable volumes like CD-ROMs here */
-          if (!thunar_g_volume_is_removable (lp->data))
-            {
-              g_object_unref (lp->data);
-              continue;
-            }
-
-          /* generate a unique name and tooltip for the volume */
-          volume_name = g_volume_get_name (lp->data);
+          /* generate a unique name and tooltip for the device */
+          device_name = thunar_device_get_name (lp->data);
           name = g_strdup_printf ("thunar-launcher-sendto%d-%p", n, launcher);
           tooltip = g_strdup_printf (ngettext ("Send the selected file to \"%s\"",
                                                "Send the selected files to \"%s\"",
-                                               n_selected_files), volume_name);
+                                               n_selected_files), device_name);
 
-          /* allocate a new action for the volume */
-          action = gtk_action_new (name, volume_name, tooltip, NULL);
+          /* allocate a new action for the device */
+          action = gtk_action_new (name, device_name, tooltip, NULL);
           g_object_set_qdata_full (G_OBJECT (action), thunar_launcher_handler_quark, lp->data, g_object_unref);
-          g_signal_connect (G_OBJECT (action), "activate", G_CALLBACK (thunar_launcher_action_sendto_volume), launcher);
+          g_signal_connect (G_OBJECT (action), "activate", G_CALLBACK (thunar_launcher_action_sendto_device), launcher);
           gtk_action_group_add_action (launcher->action_group, action);
           gtk_ui_manager_add_ui (launcher->ui_manager, launcher->ui_addons_merge_id,
-                                 "/main-menu/file-menu/sendto-menu/placeholder-sendto-actions",
-                                 name, name, GTK_UI_MANAGER_MENUITEM, FALSE);
+                                 file_menu_path, name, name, GTK_UI_MANAGER_MENUITEM, FALSE);
           gtk_ui_manager_add_ui (launcher->ui_manager, launcher->ui_addons_merge_id,
-                                 "/file-context-menu/sendto-menu/placeholder-sendto-actions",
-                                 name, name, GTK_UI_MANAGER_MENUITEM, FALSE);
+                                 context_menu_path, name, name, GTK_UI_MANAGER_MENUITEM, FALSE);
           g_object_unref (action);
 
-          /* FIXME There's no API for creating GtkActions using GIcon in GTK+ 2.14. A "gicon" property
-           * has been added to GtkAction in GTK+ 2.16 though. For now, this hack will have to do: */
-
-          ui_path = g_strconcat ("/main-menu/file-menu/sendto-menu/placeholder-sendto-actions/", name, NULL);
-          menu_item = gtk_ui_manager_get_widget (launcher->ui_manager, ui_path);
-          image = gtk_image_menu_item_get_image (GTK_IMAGE_MENU_ITEM (menu_item));
-          icon = g_volume_get_icon (lp->data);
-          gtk_image_set_from_gicon (GTK_IMAGE (image), icon, GTK_ICON_SIZE_MENU);
-          g_object_unref (icon);
-          g_free (ui_path);
-          	
-          ui_path = g_strconcat ("/file-context-menu/sendto-menu/placeholder-sendto-actions/", name, NULL);
-          menu_item = gtk_ui_manager_get_widget (launcher->ui_manager, ui_path);
-          image = gtk_image_menu_item_get_image (GTK_IMAGE_MENU_ITEM (menu_item));
-          icon = g_volume_get_icon (lp->data);
-          gtk_image_set_from_gicon (GTK_IMAGE (image), icon, GTK_ICON_SIZE_MENU);
-          g_object_unref (icon);
-          g_free (ui_path);
-
-          /* FIXME End of the hack */
+          icon = thunar_device_get_icon (lp->data);
+          if (G_LIKELY (icon != NULL))
+            {
+              gtk_action_set_gicon (action, icon);
+              g_object_unref (icon);
+            }
 
           /* cleanup */
           g_free (name);
           g_free (tooltip);
-          g_free (volume_name);
+          g_free (device_name);
         }
 
-      /* free the volumes list */
-      g_list_free (volumes);
+      /* free the devices list */
+      g_list_free (devices);
 
       /* determine the sendto handlers for the selected files */
       handlers = thunar_sendto_model_get_matching (launcher->sendto_model, launcher->selected_files);
       if (G_LIKELY (handlers != NULL))
         {
+          if (got_devices)
+            {
+              /* add separator between the devices and actions action */
+              gtk_ui_manager_add_ui (launcher->ui_manager, launcher->ui_addons_merge_id,
+                                     file_menu_path, "separator", NULL,
+                                     GTK_UI_MANAGER_SEPARATOR, FALSE);
+              gtk_ui_manager_add_ui (launcher->ui_manager, launcher->ui_addons_merge_id,
+                                     context_menu_path, "separator", NULL,
+                                     GTK_UI_MANAGER_SEPARATOR, FALSE);
+            }
+
           /* add all handlers to the user interface */
           for (lp = handlers; lp != NULL; lp = lp->next, ++n)
             {
@@ -1597,34 +1578,16 @@ thunar_launcher_sendto_idle (gpointer user_data)
 
               /* allocate a new action for the handler */
               action = gtk_action_new (name, label, tooltip, NULL);
+              gtk_action_set_gicon (action, g_app_info_get_icon (lp->data));
               g_object_set_qdata_full (G_OBJECT (action), thunar_launcher_handler_quark, lp->data, g_object_unref);
               g_signal_connect (G_OBJECT (action), "activate", G_CALLBACK (thunar_launcher_action_open), launcher);
               gtk_action_group_add_action (launcher->action_group, action);
               gtk_ui_manager_add_ui (launcher->ui_manager, launcher->ui_addons_merge_id,
-                                     "/main-menu/file-menu/sendto-menu/placeholder-sendto-actions",
-                                     name, name, GTK_UI_MANAGER_MENUITEM, FALSE);
+                                     file_menu_path, name, name, GTK_UI_MANAGER_MENUITEM, FALSE);
               gtk_ui_manager_add_ui (launcher->ui_manager, launcher->ui_addons_merge_id,
-                                     "/file-context-menu/sendto-menu/placeholder-sendto-actions",
-                                     name, name, GTK_UI_MANAGER_MENUITEM, FALSE);
+                                     context_menu_path, name, name, GTK_UI_MANAGER_MENUITEM, FALSE);
               g_object_unref (G_OBJECT (action));
 
-              /* FIXME There's no API for creating GtkActions using GIcon in GTK+ 2.14. A "gicon" property
-               * has been added to GtkAction in GTK+ 2.16 though. For now, this hack will have to do: */
-
-              ui_path = g_strconcat ("/main-menu/file-menu/sendto-menu/placeholder-sendto-actions/", name, NULL);
-              menu_item = gtk_ui_manager_get_widget (launcher->ui_manager, ui_path);
-              image = gtk_image_menu_item_get_image (GTK_IMAGE_MENU_ITEM (menu_item));
-              gtk_image_set_from_gicon (GTK_IMAGE (image), g_app_info_get_icon (lp->data), GTK_ICON_SIZE_MENU);
-              g_free (ui_path);
-              	
-              ui_path = g_strconcat ("/file-context-menu/sendto-menu/placeholder-sendto-actions/", name, NULL);
-              menu_item = gtk_ui_manager_get_widget (launcher->ui_manager, ui_path);
-              image = gtk_image_menu_item_get_image (GTK_IMAGE_MENU_ITEM (menu_item));
-              gtk_image_set_from_gicon (GTK_IMAGE (image), g_app_info_get_icon (lp->data), GTK_ICON_SIZE_MENU);
-              g_free (ui_path);
-
-              /* FIXME End of the hack */
-
               /* cleanup */
               g_free (tooltip);
               g_free (name);
@@ -1673,7 +1636,7 @@ thunar_launcher_new (void)
  *
  * Return value: the widget associated with @launcher.
  **/
-GtkWidget*
+static GtkWidget*
 thunar_launcher_get_widget (const ThunarLauncher *launcher)
 {
   _thunar_return_val_if_fail (THUNAR_IS_LAUNCHER (launcher), NULL);
