@@ -1,4 +1,4 @@
-/* $Id: thunar-shortcuts-pane.c 26421 2007-12-02 13:46:28Z benny $ */
+/* $Id$ */
 /*-
  * Copyright (c) 2005-2006 Benedikt Meurer <benny@xfce.org>
  *
@@ -43,11 +43,9 @@ enum
 
 
 
-static void          thunar_shortcuts_pane_class_init            (ThunarShortcutsPaneClass *klass);
 static void          thunar_shortcuts_pane_component_init        (ThunarComponentIface     *iface);
 static void          thunar_shortcuts_pane_navigator_init        (ThunarNavigatorIface     *iface);
 static void          thunar_shortcuts_pane_side_pane_init        (ThunarSidePaneIface      *iface);
-static void          thunar_shortcuts_pane_init                  (ThunarShortcutsPane      *shortcuts_pane);
 static void          thunar_shortcuts_pane_dispose               (GObject                  *object);
 static void          thunar_shortcuts_pane_finalize              (GObject                  *object);
 static void          thunar_shortcuts_pane_get_property          (GObject                  *object,
@@ -98,60 +96,12 @@ static const GtkActionEntry action_entries[] =
   { "sendto-shortcuts", THUNAR_STOCK_SHORTCUTS, "", NULL, NULL, G_CALLBACK (thunar_shortcuts_pane_action_shortcuts_add), },
 };
 
-static GObjectClass *thunar_shortcuts_pane_parent_class;
 
 
-
-GType
-thunar_shortcuts_pane_get_type (void)
-{
-  static GType type = G_TYPE_INVALID;
-
-  if (G_UNLIKELY (type == G_TYPE_INVALID))
-    {
-      static const GTypeInfo info =
-      {
-        sizeof (ThunarShortcutsPaneClass),
-        NULL,
-        NULL,
-        (GClassInitFunc) thunar_shortcuts_pane_class_init,
-        NULL,
-        NULL,
-        sizeof (ThunarShortcutsPane),
-        0,
-        (GInstanceInitFunc) thunar_shortcuts_pane_init,
-        NULL,
-      };
-
-      static const GInterfaceInfo component_info =
-      {
-        (GInterfaceInitFunc) thunar_shortcuts_pane_component_init,
-        NULL,
-        NULL,
-      };
-
-      static const GInterfaceInfo navigator_info =
-      {
-        (GInterfaceInitFunc) thunar_shortcuts_pane_navigator_init,
-        NULL,
-        NULL,
-      };
-
-      static const GInterfaceInfo side_pane_info =
-      {
-        (GInterfaceInitFunc) thunar_shortcuts_pane_side_pane_init,
-        NULL,
-        NULL,
-      };
-
-      type = g_type_register_static (GTK_TYPE_SCROLLED_WINDOW, I_("ThunarShortcutsPane"), &info, 0);
-      g_type_add_interface_static (type, THUNAR_TYPE_NAVIGATOR, &navigator_info);
-      g_type_add_interface_static (type, THUNAR_TYPE_COMPONENT, &component_info);
-      g_type_add_interface_static (type, THUNAR_TYPE_SIDE_PANE, &side_pane_info);
-    }
-
-  return type;
-}
+G_DEFINE_TYPE_WITH_CODE (ThunarShortcutsPane, thunar_shortcuts_pane, GTK_TYPE_SCROLLED_WINDOW,
+    G_IMPLEMENT_INTERFACE (THUNAR_TYPE_NAVIGATOR, thunar_shortcuts_pane_navigator_init)
+    G_IMPLEMENT_INTERFACE (THUNAR_TYPE_COMPONENT, thunar_shortcuts_pane_component_init)
+    G_IMPLEMENT_INTERFACE (THUNAR_TYPE_SIDE_PANE, thunar_shortcuts_pane_side_pane_init))
 
 
 
@@ -160,9 +110,6 @@ thunar_shortcuts_pane_class_init (ThunarShortcutsPaneClass *klass)
 {
   GObjectClass *gobject_class;
 
-  /* determine the parent type class */
-  thunar_shortcuts_pane_parent_class = g_type_class_peek_parent (klass);
-
   gobject_class = G_OBJECT_CLASS (klass);
   gobject_class->dispose = thunar_shortcuts_pane_dispose;
   gobject_class->finalize = thunar_shortcuts_pane_finalize;
@@ -378,6 +325,7 @@ thunar_shortcuts_pane_set_selected_files (ThunarComponent *component,
 {
   ThunarShortcutsPane *shortcuts_pane = THUNAR_SHORTCUTS_PANE (component);
   GtkTreeModel        *model;
+  GtkTreeModel        *child_model;
   GtkTreeIter          iter;
   GtkAction           *action;
   GList               *lp;
@@ -403,8 +351,9 @@ thunar_shortcuts_pane_set_selected_files (ThunarComponent *component,
       if (G_LIKELY (model != NULL))
         {
           /* check all selected folders */
+          child_model = gtk_tree_model_filter_get_model (GTK_TREE_MODEL_FILTER (model));
           for (lp = selected_files; lp != NULL; lp = lp->next)
-            if (!thunar_shortcuts_model_iter_for_file (THUNAR_SHORTCUTS_MODEL (model), lp->data, &iter))
+            if (!thunar_shortcuts_model_iter_for_file (THUNAR_SHORTCUTS_MODEL (child_model), lp->data, &iter))
               break;
         }
 
@@ -489,6 +438,7 @@ thunar_shortcuts_pane_action_shortcuts_add (GtkAction           *action,
                                             ThunarShortcutsPane *shortcuts_pane)
 {
   GtkTreeModel *model;
+  GtkTreeModel *child_model;
   GtkTreePath  *path;
   GList        *lp;
 
@@ -500,12 +450,13 @@ thunar_shortcuts_pane_action_shortcuts_add (GtkAction           *action,
   if (G_LIKELY (model != NULL))
     {
       /* add all selected folders to the model */
+      child_model = gtk_tree_model_filter_get_model (GTK_TREE_MODEL_FILTER (model));
       for (lp = shortcuts_pane->selected_files; lp != NULL; lp = lp->next)
         if (G_LIKELY (thunar_file_is_directory (lp->data)))
           {
             /* append the folder to the shortcuts model */
-            path = gtk_tree_path_new_from_indices (gtk_tree_model_iter_n_children (model, NULL), -1);
-            thunar_shortcuts_model_add (THUNAR_SHORTCUTS_MODEL (model), path, lp->data);
+            path = gtk_tree_path_new_from_indices (gtk_tree_model_iter_n_children (child_model, NULL), -1);
+            thunar_shortcuts_model_add (THUNAR_SHORTCUTS_MODEL (child_model), path, lp->data);
             gtk_tree_path_free (path);
           }
 
@@ -515,21 +466,3 @@ thunar_shortcuts_pane_action_shortcuts_add (GtkAction           *action,
       thunar_file_list_free (lp);
     }
 }
-
-
-
-/**
- * thunar_shortcuts_pane_new:
- *
- * Allocates a new #ThunarShortcutsPane instance.
- *
- * Return value: the newly allocated #ThunarShortcutsPane instance.
- **/
-GtkWidget*
-thunar_shortcuts_pane_new (void)
-{
-  return g_object_new (THUNAR_TYPE_SHORTCUTS_PANE, NULL);
-}
-
-
-
