@@ -1,21 +1,22 @@
-/* $Id$ */
+/* vi:set et ai sw=2 sts=2 ts=2: */
 /*-
  * Copyright (c) 2006-2007 Benedikt Meurer <benny@xfce.org>
- * Copyright (c) 2009 Jannis Pohlmann <jannis@xfce.org>
+ * Copyright (c) 2009-2010 Jannis Pohlmann <jannis@xfce.org>
  *
- * This program is free software; you can redistribute it and/or modify it
- * under the terms of the GNU General Public License as published by the Free
- * Software Foundation; either version 2 of the License, or (at your option)
- * any later version.
+ * This program is free software; you can redistribute it and/or 
+ * modify it under the terms of the GNU General Public License as
+ * published by the Free Software Foundation; either version 2 of 
+ * the License, or (at your option) any later version.
  *
- * This program is distributed in the hope that it will be useful, but WITHOUT
- * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
- * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
- * more details.
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the 
+ * GNU General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License along with
- * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
- * Place, Suite 330, Boston, MA  02111-1307  USA
+ * You should have received a copy of the GNU General Public 
+ * License along with this program; if not, write to the Free 
+ * Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+ * Boston, MA 02110-1301, USA.
  */
 
 #ifdef HAVE_CONFIG_H
@@ -56,15 +57,19 @@
 #include <thunar/thunar-private.h>
 #include <thunar/thunar-util.h>
 
+#include <glib.h>
+#include <glib/gstdio.h>
 
 
 /**
  * thunar_util_expand_filename:
- * @filename : a local filename.
- * @error    : return location for errors or %NULL.
+ * @filename          : a local filename.
+ * @working_directory : #GFile of the current working directory.
+ * @error             : return location for errors or %NULL.
  *
  * Takes a user-typed @filename and expands a tilde at the
- * beginning of the @filename.
+ * beginning of the @filename. It also resolves paths prefixed with
+ * '.' using the current working directory.
  *
  * The caller is responsible to free the returned string using
  * g_free() when no longer needed.
@@ -73,6 +78,7 @@
  **/
 gchar *
 thunar_util_expand_filename (const gchar  *filename,
+                             GFile        *working_directory,
                              GError      **error)
 {
   struct passwd *passwd;
@@ -80,6 +86,8 @@ thunar_util_expand_filename (const gchar  *filename,
   const gchar   *remainder;
   const gchar   *slash;
   gchar         *username;
+  gchar         *pwd;
+  gchar         *result = NULL;
 
   g_return_val_if_fail (filename != NULL, NULL);
 
@@ -91,48 +99,75 @@ thunar_util_expand_filename (const gchar  *filename,
     }
 
   /* check if we start with a '~' */
-  if (G_LIKELY (*filename != '~'))
-    return g_strdup (filename);
-
-  /* examine the remainder of the filename */
-  remainder = filename + 1;
-
-  /* if we have only the slash, then we want the home dir */
-  if (G_UNLIKELY (*remainder == '\0'))
-    return g_strdup (xfce_get_homedir ());
-
-  /* lookup the slash */
-  for (slash = remainder; *slash != '\0' && *slash != G_DIR_SEPARATOR; ++slash)
-    ;
-
-  /* check if a username was given after the '~' */
-  if (G_LIKELY (slash == remainder))
+  if (*filename == '~')
     {
-      /* replace the tilde with the home dir */
-      replacement = xfce_get_homedir ();
-    }
-  else
-    {
-      /* lookup the pwd entry for the username */
-      username = g_strndup (remainder, slash - remainder);
-      passwd = getpwnam (username);
-      g_free (username);
+      /* examine the remainder of the filename */
+      remainder = filename + 1;
 
-      /* check if we have a valid entry */
-      if (G_UNLIKELY (passwd == NULL))
+      /* if we have only the slash, then we want the home dir */
+      if (G_UNLIKELY (*remainder == '\0'))
+        return g_strdup (xfce_get_homedir ());
+
+      /* lookup the slash */
+      for (slash = remainder; *slash != '\0' && *slash != G_DIR_SEPARATOR; ++slash);
+
+      /* check if a username was given after the '~' */
+      if (G_LIKELY (slash == remainder))
         {
-          username = g_strndup (remainder, slash - remainder);
-          g_set_error (error, G_FILE_ERROR, G_FILE_ERROR_INVAL, _("Unknown user \"%s\""), username);
-          g_free (username);
-          return NULL;
+          /* replace the tilde with the home dir */
+          replacement = xfce_get_homedir ();
         }
+      else
+        {
+          /* lookup the pwd entry for the username */
+          username = g_strndup (remainder, slash - remainder);
+          passwd = getpwnam (username);
+          g_free (username);
 
-      /* use the homedir of the specified user */
-      replacement = passwd->pw_dir;
+          /* check if we have a valid entry */
+          if (G_UNLIKELY (passwd == NULL))
+            {
+              username = g_strndup (remainder, slash - remainder);
+              g_set_error (error, G_FILE_ERROR, G_FILE_ERROR_INVAL, _("Unknown user \"%s\""), username);
+              g_free (username);
+              return NULL;
+            }
+
+          /* use the homedir of the specified user */
+          replacement = passwd->pw_dir;
+        }
+    
+      /* generate the filename */
+      return g_build_filename (replacement, slash, NULL);
+    }
+  else if (*filename == '.')
+    {
+      /* examine the remainder of the filename */
+      remainder = filename + 1;
+      
+      /* transform working directory into a filename string */
+      if (G_LIKELY (working_directory != NULL))
+        {
+          pwd = g_file_get_path (working_directory);
+    
+          /* if we only have the slash then we want the working directory only */
+          if (G_UNLIKELY (*remainder == '\0'))
+            return pwd;
+
+          /* concatenate working directory and remainder */
+          result = g_build_filename (pwd, remainder, G_DIR_SEPARATOR_S, NULL);
+
+          /* free the working directory string */
+          g_free (pwd);
+        }
+      else
+        result = g_strdup (filename);
+
+      /* return the resulting path string */
+      return result;
     }
 
-  /* generate the filename */
-  return g_build_filename (replacement, slash, NULL);
+  return g_strdup (filename);
 }
 
 
@@ -274,7 +309,7 @@ thunar_util_parse_parent (gpointer    parent,
     {
       /* parent is a widget, so let's determine the toplevel window */
       window = gtk_widget_get_toplevel (GTK_WIDGET (parent));
-      if (GTK_WIDGET_TOPLEVEL (window))
+      if (gtk_widget_is_toplevel (window))
         {
           /* make sure the toplevel window is shown */
           gtk_widget_show_now (window);
@@ -371,27 +406,11 @@ thunar_util_change_working_directory (const gchar *new_directory)
 
   _thunar_return_val_if_fail (new_directory != NULL && *new_directory != '\0', NULL);
 
-  /* allocate a path buffer for the old working directory */
-  old_directory = g_malloc0 (sizeof (gchar) * MAXPATHLEN);
-
   /* try to determine the current working directory */
-#ifdef G_PLATFORM_WIN32
-  if (_getcwd (old_directory, MAXPATHLEN) == NULL)
-#else
-  if (getcwd (old_directory, MAXPATHLEN) == NULL)
-#endif
-    {
-      /* working directory couldn't be determined, reset the buffer */
-      g_free (old_directory);
-      old_directory = NULL;
-    }
+  old_directory = g_get_current_dir();
 
   /* try switching to the new working directory */
-#ifdef G_PLATFORM_WIN32
-  if (_chdir (new_directory) != 0)
-#else
-  if (chdir (new_directory) != 0)
-#endif
+  if (g_chdir (new_directory) != 0)
     {
       /* switching failed, we don't need to return the old directory */
       g_free (old_directory);
